/***************************************************************************
 *
 * algorithm.cc - Non-inline definitions for the Standard Library algorithms
 *
 ***************************************************************************
 *
 * Copyright (c) 1994
 * Hewlett-Packard Company
 *
 * Permission to use, copy, modify, distribute and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear
 * in supporting documentation.  Hewlett-Packard Company makes no
 * representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 *
 ***************************************************************************
 *
 * Copyright (c) 1994-1999 Rogue Wave Software, Inc.  All Rights Reserved.
 *
 * This computer software is owned by Rogue Wave Software, Inc. and is
 * protected by U.S. copyright laws and other laws and by international
 * treaties.  This computer software is furnished by Rogue Wave Software,
 * Inc. pursuant to a written license agreement and may be used, copied,
 * transmitted, and stored only in accordance with the terms of such
 * license and with the inclusion of the above copyright notice.  This
 * computer software or any other copies thereof may not be provided or
 * otherwise made available to any other person.
 *
 * U.S. Government Restricted Rights.  This computer software is provided
 * with Restricted Rights.  Use, duplication, or disclosure by the
 * Government is subject to restrictions as set forth in subparagraph (c)
 * (1) (ii) of The Rights in Technical Data and Computer Software clause
 * at DFARS 252.227-7013 or subparagraphs (c) (1) and (2) of the
 * Commercial Computer Software  Restricted Rights at 48 CFR 52.227-19,
 * as applicable.  Manufacturer is Rogue Wave Software, Inc., 5500
 * Flatiron Parkway, Boulder, Colorado 80301 USA.
 *
 **************************************************************************/

#include <stdcomp.h>

#pragma disable_warn

#ifndef _RWSTD_NO_NAMESPACE
namespace std {
#endif

//
// Forward declare raw_storage_iterator 
//   
  template <class _OutputIterator, class _T>
  class raw_storage_iterator;

//
// Non-modifying sequence operations.
//

  template <class _InputIterator, class _Function>
  _Function for_each (_InputIterator first, _InputIterator last, _Function f)
  {
    while (first != last) f(*first++);
    return f;
  }

  template <class _InputIterator, class _T>
  _InputIterator find (_InputIterator first, _InputIterator last, const _T& value)
  {
    while (first != last && !(*first == value))
      ++first;
    return first;
  }

  template <class _InputIterator, class _Predicate>
  _InputIterator find_if (_InputIterator first, _InputIterator last, _Predicate pred)
  {
    while (first != last && !pred(*first)) ++first;
    return first;
  }


  template <class _ForwardIterator1, class _ForwardIterator2, 
  class _Distance>
  _ForwardIterator1 __find_end (_ForwardIterator1 first1,
                               _ForwardIterator1 last1,
                               _ForwardIterator2 first2,
                               _ForwardIterator2 last2,
                               _Distance*)
  {
    _Distance d, d2;
    __initialize(d,_Distance(0));
    __initialize(d2,_Distance(0));
    distance(first2,last2,d);
    if (!d)
      return last1;
    distance(first1,last1,d2);
    _ForwardIterator1 save = last1;
 
    while (d2 >= d)
    {
      if (equal(first2,last2,first1))
        save  = first1;
      __initialize(d2,_Distance(0));
      distance(++first1,last1,d2);
    }
    return save;
  }


  template <class _ForwardIterator1, class _ForwardIterator2>
  _ForwardIterator1 find_end (_ForwardIterator1 first1,
                             _ForwardIterator1 last1,
                             _ForwardIterator2 first2,
                             _ForwardIterator2 last2)
  {
    return __find_end(first1,last1,first2,last2,
                      __distance_type(first1));
  }

  template <class _ForwardIterator1, class _ForwardIterator2, 
  class _BinaryPredicate, class _Distance>
  _ForwardIterator1 __find_end (_ForwardIterator1 first1,
                               _ForwardIterator1 last1,
                               _ForwardIterator2 first2,
                               _ForwardIterator2 last2,
                               _BinaryPredicate pred,
                               _Distance*)
  {
    _Distance d, d2;
    __initialize(d,_Distance(0));
    __initialize(d2,_Distance(0));
    distance(first2,last2,d);
    if (!d)
      return last1;
    distance(first1,last1,d2);
    _ForwardIterator1 save = last1;
 
    while (d2 >= d)
    {
      if (equal(first2,last2,first1,pred))
        save  = first1;
      __initialize(d2,_Distance(0));
      distance(++first1,last1,d2);
    }
    return save;
  }

  template <class _ForwardIterator1, class _ForwardIterator2, 
  class _BinaryPredicate>
  _ForwardIterator1 find_end (_ForwardIterator1 first1,
                             _ForwardIterator1 last1,
                             _ForwardIterator2 first2,
                             _ForwardIterator2 last2,
                             _BinaryPredicate pred)
  {
    return __find_end(first1,last1,first2,last2,
                      pred,__distance_type(first1));
  }


  template <class _ForwardIterator1, class _ForwardIterator2>
  _ForwardIterator1 find_first_of (_ForwardIterator1 first1, _ForwardIterator1 last1,
                                  _ForwardIterator2 first2, _ForwardIterator2 last2)
  {
    if (first2 == last2)
      return last1;
    _ForwardIterator1 next = first1;
    while (next != last1)
    {
      if (find(first2,last2,*next) != last2)
        return next;
      next++;
    }
    return last1;
  }

  template <class _ForwardIterator1, class _ForwardIterator2,
  class _BinaryPredicate>
  _ForwardIterator1 find_first_of (_ForwardIterator1 first1,_ForwardIterator1 last1,
                                  _ForwardIterator2 first2,_ForwardIterator2 last2,
                                  _BinaryPredicate pred)
  {
    if (first2 == last2)
      return last1;
    
    for (_ForwardIterator1 next = first1; next != last1; ++next)
      for (_ForwardIterator2 iter = first2; iter != last2; ++iter)
        if (pred(*next, *iter))
          return next;
    return last1;
  }


  template <class _ForwardIterator>
  _ForwardIterator adjacent_find (_ForwardIterator first, _ForwardIterator last)
  {
    if (first == last) return last;
    _ForwardIterator next = first;
    while (++next != last)
    {
      if (*first == *next) return first;
      first = next;
    }
    return last;
  }

  template <class _ForwardIterator, class _BinaryPredicate>
  _ForwardIterator adjacent_find (_ForwardIterator first, _ForwardIterator last,
                                 _BinaryPredicate binary_pred)
  {
    if (first == last) return last;
    _ForwardIterator next = first;
    while (++next != last)
    {
      if (binary_pred(*first, *next)) return first;
      first = next;
    }
    return last;
  }

#ifndef _RWSTD_NO_CLASS_PARTIAL_SPEC
  template <class _InputIterator, class _T>
  _TYPENAME iterator_traits<_InputIterator>::difference_type
  count (_InputIterator first, _InputIterator last, const _T& value)
  {
    _TYPENAME iterator_traits<_InputIterator>::difference_type n = 0;
    while (first != last) 
      if (*first++ == value) ++n;
    return n;
  }

  template <class _InputIterator, class _Predicate>
  _TYPENAME iterator_traits<_InputIterator>::difference_type
  count_if (_InputIterator first, _InputIterator last, _Predicate pred)
  {
    _TYPENAME iterator_traits<_InputIterator>::difference_type n = 0;
    while (first != last)
      if (pred(*first++)) ++n;
    return n;
  }
#endif /* _RWSTD_NO_CLASS_PARTIAL_SPEC */

#ifndef _RWSTD_NO_OLD_COUNT
  template <class _InputIterator, class _T, class _Size>
  void count (_InputIterator first, _InputIterator last, const _T& value, _Size& n)
  {
    while (first != last) 
      if (*first++ == value) ++n;
  }

  template <class _InputIterator, class _Predicate, class _Size>
  void count_if (_InputIterator first, _InputIterator last, _Predicate pred, 
                 _Size& n)
  {
    while (first != last)
      if (pred(*first++)) ++n;
  }
#endif /* _RWSTD_NO_OLD_COUNT */

  template <class _InputIterator1, class _InputIterator2>
  pair<_InputIterator1, _InputIterator2> mismatch(_InputIterator1 first1,
                                                _InputIterator1 last1,
                                                _InputIterator2 first2)
  {
    while (first1 != last1 && *first1 == *first2)
    {
      ++first1;
      ++first2;
    }
    pair<_InputIterator1, _InputIterator2> tmp(first1, first2);
    return tmp;
  }

  template <class _InputIterator1, class _InputIterator2, class _BinaryPredicate>
  pair<_InputIterator1, _InputIterator2> mismatch (_InputIterator1 first1,
                                                 _InputIterator1 last1,
                                                 _InputIterator2 first2,
                                                 _BinaryPredicate binary_pred)
  {
    while (first1 != last1 && binary_pred(*first1, *first2))
    {
      ++first1;
      ++first2;
    }
    pair<_InputIterator1, _InputIterator2> tmp(first1, first2);
    return tmp;
  }

  template <class _ForwardIterator1, class _ForwardIterator2,
  class _Distance1, class _Distance2>
  _ForwardIterator1 __search (_ForwardIterator1 first1, _ForwardIterator1 last1,
                             _ForwardIterator2 first2, _ForwardIterator2 last2,
                             _Distance1*, _Distance2*)
  {
    _Distance1 d1;
    __initialize(d1, _Distance1(0));
    distance(first1, last1, d1);
    _Distance2 d2;
    __initialize(d2, _Distance2(0));
    distance(first2, last2, d2);

    if (d1 < d2) return last1;

    _ForwardIterator1 current1 = first1;
    _ForwardIterator2 current2 = first2;

    while (current2 != last2)
    {
      if (!(*current1++ == *current2++))
        if (d1-- == d2)
          return last1;
        else
        {
          current1 = ++first1;
          current2 = first2;
        }
    }
    return (current2 == last2) ? first1 : last1;
  }

  template <class _ForwardIterator1, class _ForwardIterator2,
  class _BinaryPredicate, class _Distance1, class _Distance2>
  _ForwardIterator1 __search (_ForwardIterator1 first1, _ForwardIterator1 last1,
                             _ForwardIterator2 first2, _ForwardIterator2 last2,
                             _BinaryPredicate binary_pred, _Distance1*, _Distance2*)
  {
    _Distance1 d1;
    __initialize(d1, _Distance1(0));
    distance(first1, last1, d1);
    _Distance2 d2;
    __initialize(d2, _Distance2(0));
    distance(first2, last2, d2);

    if (d1 < d2) return last1;

    _ForwardIterator1 current1 = first1;
    _ForwardIterator2 current2 = first2;

    while (current2 != last2)
    {
      if (!binary_pred(*current1++, *current2++))
        if (d1-- == d2)
          return last1;
        else
        {
          current1 = ++first1;
          current2 = first2;
        }
    }
    return (current2 == last2) ? first1 : last1;
  }

  template <class _ForwardIterator, class _Distance, class _Size, class _T>
  _ForwardIterator __search_n (_ForwardIterator first, _ForwardIterator last,
                              _Distance*, _Size count, const _T& value)
  {
    _Distance d;
    __initialize(d, _Distance(0));
    distance(first, last, d);

    if (d < count || count <= 0) return last;

    _Distance        span    = d - count;
    _Size            matches = 0;
    _ForwardIterator current = first;

    while (current != last)
    {
      if (!(*current++ == value))
      {
        if (span < matches + 1)
          return last;
        span   -= matches + 1;
        matches = 0;
        first   = current;
      }
      else
        if (++matches == count)
          return first;
    }

    return last;
  }

  template <class _ForwardIterator, class _Distance, class _Size, class _T,
  class _BinaryPredicate>
  _ForwardIterator __search_n (_ForwardIterator first, _ForwardIterator last,
                              _Distance*, _Size count, const _T& value,
                              _BinaryPredicate pred)
  {
    _Distance d;
    __initialize(d, _Distance(0));
    distance(first, last, d);

    if (d < count || count <= 0) return last;

    _Distance        span    = d - count;
    _Size            matches = 0;
    _ForwardIterator current = first;

    while (current != last)
    {
      if (!pred(*current++, value))
      {
        if (span < matches + 1)
          return last;
        span   -= matches + 1;
        matches = 0;
        first   = current;
      }
      else
        if (++matches == count)
          return first;
    }

    return last;
  }

//
// Modifying sequence operations.
//

  template <class _InputIterator, class _OutputIterator>
  _OutputIterator copy (_InputIterator first, _InputIterator last,
                       _OutputIterator result)
  {
    while (first != last) *result++ = *first++;
    return result;
  }

  template <class _BidirectionalIterator1, class _BidirectionalIterator2>
  _BidirectionalIterator2 copy_backward (_BidirectionalIterator1 first, 
                                        _BidirectionalIterator1 last, 
                                        _BidirectionalIterator2 result)
  {
    while (first != last) *--result = *--last;
    return result;
  }

  template <class _ForwardIterator1, class _ForwardIterator2>
  _ForwardIterator2 swap_ranges (_ForwardIterator1 first1, _ForwardIterator1 last1,
                                _ForwardIterator2 first2)
  {
    while (first1 != last1) iter_swap(first1++, first2++);
    return first2;
  }

  template <class _InputIterator, class _OutputIterator, class _UnaryOperation>
  _OutputIterator transform (_InputIterator first, _InputIterator last,
                            _OutputIterator result, _UnaryOperation op)
  {
    while (first != last) *result++ = op(*first++);
    return result;
  }

  template <class _InputIterator1, class _InputIterator2, class _OutputIterator,
  class BinaryOperation>
  _OutputIterator transform (_InputIterator1 first1, _InputIterator1 last1,
                            _InputIterator2 first2, _OutputIterator result,
                            BinaryOperation binary_op)
  {
    while (first1 != last1) *result++ = binary_op(*first1++, *first2++);
    return result;
  }

  template <class _ForwardIterator, class _T>
  void replace (_ForwardIterator first, _ForwardIterator last, const _T& old_value,
                const _T& new_value)
  {
    while (first != last)
    {
      if (*first == old_value) *first = new_value;
      ++first;
    }
  }

  template <class _ForwardIterator, class _Predicate, class _T>
  void replace_if (_ForwardIterator first, _ForwardIterator last, _Predicate pred,
                   const _T& new_value)
  {
    while (first != last)
    {
      if (pred(*first)) *first = new_value;
      ++first;
    }
  }

  template <class _InputIterator, class _OutputIterator, class _T>
  _OutputIterator replace_copy (_InputIterator first, _InputIterator last,
                               _OutputIterator result, const _T& old_value,
                               const _T& new_value)
  {
    while (first != last)
    {
      *result++ = *first == old_value ? new_value : *first;
      ++first;
    }
    return result;
  }

  template <class _Iterator, class _OutputIterator, class _Predicate, class _T>
  _OutputIterator replace_copy_if (_Iterator first, _Iterator last,
                                  _OutputIterator result, _Predicate pred,
                                  const _T& new_value)
  {
    while (first != last)
    {
      if(pred(*first))
        *result++ = new_value;
      else
        *result++ = *first;
      ++first;
    }
    return result;
  }

  template <class _ForwardIterator, class _T>
#ifdef _RWSTD_FILL_NAME_CLASH
  void std_fill (_ForwardIterator first, _ForwardIterator last, const _T& value)
#else
  void fill (_ForwardIterator first, _ForwardIterator last, const _T& value)
#endif
  {
    while (first != last) *first++ = value;
  }

  template <class _OutputIterator, class _Size, class _T>
  void fill_n (_OutputIterator first, _Size n, const _T& value)
  {
    while (n-- > 0) *first++ = value;
  }

  template <class _ForwardIterator, class _Generator>
  void generate (_ForwardIterator first, _ForwardIterator last, _Generator gen)
  {
    while (first != last) *first++ = gen();
  }

  template <class _OutputIterator, class _Size, class _Generator>
  void generate_n (_OutputIterator first, _Size n, _Generator gen)
  {
    while (n-- > 0) *first++ = gen();
  }

  template <class _InputIterator, class _OutputIterator, class _T>
  _OutputIterator remove_copy (_InputIterator first, _InputIterator last,
                              _OutputIterator result, const _T& value)
  {
    while (first != last)
    {
      if (!(*first == value)) *result++ = *first;
      ++first;
    }
    return result;
  }

  template <class _InputIterator, class _OutputIterator, class _Predicate>
  _OutputIterator remove_copy_if (_InputIterator first, _InputIterator last,
                                 _OutputIterator result, _Predicate pred)
  {
    while (first != last)
    {
      if (!pred(*first)) *result++ = *first;
      ++first;
    }
    return result;
  }

  template <class _InputIterator, class _ForwardIterator>
  _ForwardIterator __unique_copy (_InputIterator first, _InputIterator last,
                                 _ForwardIterator result, forward_iterator_tag)
  {
    *result = *first;
    while (++first != last)
      if (!(*result == *first)) *++result = *first;
    return ++result;
  }

  template <class _InputIterator, class _OutputIterator, class _T>
  _OutputIterator __unique_copy (_InputIterator first, _InputIterator last,
                                _OutputIterator result, _T*)
  {
    _T value = *first;
    *result = value;
    while (++first != last)
    {
      if (!(value == *first))
      {
        value = *first;
        *++result = value;
      }
    }
    return ++result;
  }

  template <class _InputIterator, class _ForwardIterator, class _BinaryPredicate>
  _ForwardIterator __unique_copy (_InputIterator first, _InputIterator last,
                                 _ForwardIterator result, 
                                 _BinaryPredicate binary_pred,
                                 forward_iterator_tag)
  {
    *result = *first;
    while (++first != last)
      if (!binary_pred(*result, *first)) *++result = *first;
    return ++result;
  }

  template <class _InputIterator, class _OutputIterator, class _BinaryPredicate,
  class _T>
  _OutputIterator __unique_copy (_InputIterator first, _InputIterator last,
                                _OutputIterator result,
                                _BinaryPredicate binary_pred, _T*)
  {
    _T value = *first;
    *result = value;
    while (++first != last)
    {
      if (!binary_pred(value, *first))
      {
        value = *first;
        *++result = value;
      }
    }
    return ++result;
  }

  template <class _BidirectionalIterator>
  void __reverse (_BidirectionalIterator first, _BidirectionalIterator last, 
                  bidirectional_iterator_tag)
  {
    while (true)
      if (first == last || first == --last)
        return;
      else
        iter_swap(first++, last);
  }

  template <class _RandomAccessIterator>
  void __reverse (_RandomAccessIterator first, _RandomAccessIterator last,
                  random_access_iterator_tag)
  {
    while (first < last) iter_swap(first++, --last);
  }

  template <class _BidirectionalIterator, class _OutputIterator>
  _OutputIterator reverse_copy (_BidirectionalIterator first,
                               _BidirectionalIterator last,
                               _OutputIterator result)
  {
    while (first != last) *result++ = *--last;
    return result;
  }

  template <class _ForwardIterator, class _Distance>
  void __rotate (_ForwardIterator first, _ForwardIterator middle,
                 _ForwardIterator last, _Distance*, forward_iterator_tag)
  {
    for (_ForwardIterator i = middle; ;)
    {
      iter_swap(first++, i++);
      if (first == middle)
      {
        if (i == last) return;
        middle = i;
      }
      else if (i == last)
        i = middle;
    }
  }

  template <class _EuclideanRingElement>
  _EuclideanRingElement __gcd (_EuclideanRingElement m, _EuclideanRingElement n)
  {
    while (n != 0)
    {
      _EuclideanRingElement t = m % n;
      m = n;
      n = t;
    }
    return m;
  }

  template <class _RandomAccessIterator, class _Distance, class _T>
  void __rotate_cycle (_RandomAccessIterator first, _RandomAccessIterator last,
                       _RandomAccessIterator initial, _Distance shift, _T*)
  {
    _T value = *initial;
    _RandomAccessIterator ptr1 = initial;
    _RandomAccessIterator ptr2 = ptr1 + shift;
    while (ptr2 != initial)
    {
      *ptr1 = *ptr2;
      ptr1 = ptr2;
      if (last - ptr2 > shift)
        ptr2 += shift;
      else
        ptr2 = first + (shift - (last - ptr2));
    }
    *ptr1 = value;
  }

  template <class _RandomAccessIterator, class _Distance>
  void __rotate (_RandomAccessIterator first, _RandomAccessIterator middle,
                 _RandomAccessIterator last, _Distance*,
                 random_access_iterator_tag)
  {
    _Distance n = __gcd(last - first, middle - first);
    while (n--)
      __rotate_cycle(first, last, first + n, middle - first,
                     _RWSTD_VALUE_TYPE(first));
  }


#ifndef _RWSTD_NO_NAMESPACE
}
namespace __rwstd {
#endif
  extern unsigned _RWSTDExport long __long_random (unsigned long);
#ifndef _RWSTD_NO_NAMESPACE
}
namespace std {
#endif

  template <class _RandomAccessIterator, class _Distance>
  void __random_shuffle (_RandomAccessIterator first, _RandomAccessIterator last,
                         _Distance*)
  {
    if (!(first == last))
      for (_RandomAccessIterator i = first + 1; i != last; ++i)
        iter_swap(i, first + _Distance(__RWSTD::__long_random((i - first) + 1)));
  }

  template <class _RandomAccessIterator, class _RandomNumberGenerator>
  void random_shuffle (_RandomAccessIterator first, _RandomAccessIterator last,
                       _RandomNumberGenerator& rand)
  {
    if (!(first == last))
      for (_RandomAccessIterator i = first + 1; i != last; ++i)
        iter_swap(i, first + rand((i - first) + 1));
  }

  template <class _BidirectionalIterator, class _Predicate>
  _BidirectionalIterator partition (_BidirectionalIterator first,
                                   _BidirectionalIterator last, _Predicate pred)
  {
    while (true)
    {
      while (true)
      {
        if (first == last)
          return first;
        else if (pred(*first))
          ++first;
        else
          break;
      }
      --last;
      while (true)
      {
        if (first == last)
          return first;
        else if (!pred(*last))
          --last;
        else
          break;
      }
      iter_swap(first, last);
      ++first;
    }
  }

  template <class _BidirectionalIterator, class _Predicate, class _Distance>
  _BidirectionalIterator __inplace_stable_partition (_BidirectionalIterator first,
                                                    _BidirectionalIterator last,
                                                    _Predicate pred,
                                                    _Distance len)
  {
    if (len == 1) return pred(*first) ? last : first;
    _BidirectionalIterator middle = first;
    advance(middle, len / 2);
    _BidirectionalIterator 
      first_cut = __inplace_stable_partition(first, middle, pred, len / 2);
    _BidirectionalIterator 
      second_cut = __inplace_stable_partition(middle, last, pred, len - len / 2);
    rotate(first_cut, middle, second_cut);
    __initialize(len, _Distance(0));
    distance(middle, second_cut, len);
    advance(first_cut, len);
    return first_cut;
  }

  template <class _BidirectionalIterator, class _Pointer, class _Predicate,
  class _Distance, class _T>
  _BidirectionalIterator __stable_partition_adaptive (_BidirectionalIterator first,
                                                     _BidirectionalIterator last,
                                                     _Predicate pred, _Distance len,
                                                     _Pointer buffer,
                                                     _Distance buffer_size,
                                                     _Distance& fill_pointer, _T*)
  {
    if (len <= buffer_size)
    {
      len = 0;
      _BidirectionalIterator result1 = first;
      _Pointer result2 = buffer;
      while (first != last && len < fill_pointer)
      {
        if (pred(*first))
          *result1++ = *first++;
        else
        {
          *result2++ = *first++;
          ++len;
        }
      }
      if (first != last)
      {
        raw_storage_iterator<_Pointer, _T> result3(result2);
        while (first != last)
        {
          if (pred(*first))
            *result1++ = *first++;
          else
          {
            *result3++ = *first++;
            ++len;
          }
        }
        fill_pointer = len;
      }
      copy(buffer, buffer + len, result1);
      return result1;
    }
    _BidirectionalIterator middle = first;
    advance(middle, len / 2);
    _BidirectionalIterator first_cut = __stable_partition_adaptive
      (first, middle, pred, len / 2, buffer, buffer_size, fill_pointer, (_T*)0);
    _BidirectionalIterator second_cut = __stable_partition_adaptive
      (middle, last, pred, len-len/2, buffer, buffer_size, fill_pointer, (_T*)0);
    rotate(first_cut, middle, second_cut);
    __initialize(len, _Distance(0));
    distance(middle, second_cut, len);
    advance(first_cut, len);
    return first_cut;
  }

  template <class _BidirectionalIterator, class _Predicate, class _Pointer,
            class _Distance>
  _BidirectionalIterator __stable_partition (_BidirectionalIterator first,
                                            _BidirectionalIterator last,
                                            _Predicate pred, _Distance len,
                                            pair<_Pointer, _Distance> p)
  {
    if (p.first == 0)
      return __inplace_stable_partition(first, last, pred, len);
    _Distance fill_pointer = 0;
    _BidirectionalIterator result = 
      __stable_partition_adaptive(first, last, pred, len, p.first,
                                  p.second, fill_pointer,
                                  _RWSTD_VALUE_TYPE(first)); 
    __RWSTD::__destroy(p.first, p.first + fill_pointer);
    return_temporary_buffer(p.first);
    return result;
  }

//
// Sorting and related operations.
//

  template <class _RandomAccessIterator, class _T>
  _RandomAccessIterator __unguarded_partition (_RandomAccessIterator first, 
                                              _RandomAccessIterator last, 
                                              _T pivot)
  {
    while (true)
    {
      while (*first < pivot) ++first;
      --last;
      while (pivot < *last) --last;
      if (!(first < last)) return first;
      iter_swap(first, last);
      ++first;
    }
  }    

  template <class _RandomAccessIterator, class _T, class _Compare>
  _RandomAccessIterator __unguarded_partition (_RandomAccessIterator first, 
                                              _RandomAccessIterator last, 
                                              _T pivot, _Compare _RWSTD_COMP)
  {
    while (true)
    {
      while (_RWSTD_COMP(*first, pivot)) ++first;
      --last;
      while (_RWSTD_COMP(pivot, *last)) --last;
      if (!(first < last)) return first;
      iter_swap(first, last);
      ++first;
    }
  }

  const int __stl_threshold = 16;

  template <class _RandomAccessIterator, class _T>
  void __quick_sort_loop_aux (_RandomAccessIterator first,
                              _RandomAccessIterator last, _T*)
  {
    while (last - first > __stl_threshold)
    {
      _RandomAccessIterator cut = __unguarded_partition
      (first, last, _T(__median(*first, *(first + (last - first)/2),
                               *(last - 1))));
      if (cut - first >= last - cut)
      {
        __quick_sort_loop(cut, last);
        last = cut;
      }
      else
      {
        __quick_sort_loop(first, cut);
        first = cut;
      }
    }
  }

  template <class _RandomAccessIterator, class _T, class _Compare>
  void __quick_sort_loop_aux (_RandomAccessIterator first, 
                              _RandomAccessIterator last, _T*, _Compare _RWSTD_COMP)
  {
    while (last - first > __stl_threshold)
    {
      _RandomAccessIterator cut = __unguarded_partition
      (first, last, _T(__median(*first, *(first + (last - first)/2), 
                               *(last - 1), _RWSTD_COMP)), _RWSTD_COMP);
      if (cut - first >= last - cut)
      {
        __quick_sort_loop(cut, last, _RWSTD_COMP);
        last = cut;
      }
      else
      {
        __quick_sort_loop(first, cut, _RWSTD_COMP);
        first = cut;
      }
    }
  }



  template <class _RandomAccessIterator, class _T>
  void __unguarded_linear_insert (_RandomAccessIterator last, _T value)
  {
    _RandomAccessIterator next = last;
    --next;
    while (value < *next)
    {
      *last = *next;
      last = next--;
    }
    *last = value;
  }

  template <class _RandomAccessIterator, class _T, class _Compare>
  void __unguarded_linear_insert (_RandomAccessIterator last,_T value,_Compare _RWSTD_COMP)
  {
    _RandomAccessIterator next = last;
    --next;  
    while (_RWSTD_COMP(value , *next))
    {
      *last = *next;
      last = next--;
    }
    *last = value;
  }

  template <class _RandomAccessIterator>
  void __insertion_sort (_RandomAccessIterator first, _RandomAccessIterator last)
  {
    if (!(first == last))
      for (_RandomAccessIterator i = first + 1; i != last; ++i)
        __linear_insert(first, i, _RWSTD_VALUE_TYPE(first));
  }

  template <class _RandomAccessIterator, class _Compare>
  void __insertion_sort (_RandomAccessIterator first,
                         _RandomAccessIterator last, _Compare _RWSTD_COMP)
  {
    if (!(first == last))
      for (_RandomAccessIterator i = first + 1; i != last; ++i)
        __linear_insert(first, i, _RWSTD_VALUE_TYPE(first), _RWSTD_COMP);
  }

  template <class _RandomAccessIterator, class _T>
  void __unguarded_insertion_sort_aux (_RandomAccessIterator first, 
                                       _RandomAccessIterator last, _T*)
  {
    for (_RandomAccessIterator i = first; i != last; ++i)
      __unguarded_linear_insert(i, _T(*i));
  }

  template <class _RandomAccessIterator, class _T, class _Compare>
  void __unguarded_insertion_sort_aux (_RandomAccessIterator first, 
                                       _RandomAccessIterator last,
                                       _T*, _Compare _RWSTD_COMP)
  {
    for (_RandomAccessIterator i = first; i != last; ++i)
      __unguarded_linear_insert(i, _T(*i), _RWSTD_COMP);
  }


  template <class _RandomAccessIterator>
  void __final_insertion_sort (_RandomAccessIterator first, 
                               _RandomAccessIterator last)
  {
    if (last - first > __stl_threshold)
    {
      __insertion_sort(first, first + __stl_threshold);
      __unguarded_insertion_sort(first + __stl_threshold, last);
    }
    else
      __insertion_sort(first, last);
  }

  template <class _RandomAccessIterator, class _Compare>
  void __final_insertion_sort (_RandomAccessIterator first, 
                               _RandomAccessIterator last, _Compare _RWSTD_COMP)
  {
    if (last - first > __stl_threshold)
    {
      __insertion_sort(first, first + __stl_threshold, _RWSTD_COMP);
      __unguarded_insertion_sort(first + __stl_threshold, last, _RWSTD_COMP);
    }
    else
      __insertion_sort(first, last, _RWSTD_COMP);
  }

  template <class _RandomAccessIterator1, class _RandomAccessIterator2,
  class _Distance>
  void __merge_sort_loop (_RandomAccessIterator1 first,
                          _RandomAccessIterator1 last, 
                          _RandomAccessIterator2 result, _Distance step_size)
  {
    _Distance two_step = 2 * step_size;

    while (last - first >= two_step)
    {
      result = merge(first, first + step_size,
                     first + step_size, first + two_step, result);
      first += two_step;
    }
    step_size = (min)(_Distance(last - first), step_size);

    merge(first, first + step_size, first + step_size, last, result);
  }

  template <class _RandomAccessIterator1, class _RandomAccessIterator2,
  class _Distance, class _Compare>
  void __merge_sort_loop (_RandomAccessIterator1 first,
                          _RandomAccessIterator1 last, 
                          _RandomAccessIterator2 result, _Distance step_size,
                          _Compare _RWSTD_COMP)
  {
    _Distance two_step = 2 * step_size;

    while (last - first >= two_step)
    {
      result = merge(first, first + step_size,
                     first + step_size, first + two_step, result, _RWSTD_COMP);
      first += two_step;
    }
    step_size = (min)(_Distance(last - first), step_size);

    merge(first, first + step_size, first + step_size, last, result, _RWSTD_COMP);
  }

  const int __stl_chunk_size = 7;
    
  template <class _RandomAccessIterator, class _Distance>
  void __chunk_insertion_sort (_RandomAccessIterator first, 
                               _RandomAccessIterator last, _Distance chunk_size)
  {
    while (last - first >= chunk_size)
    {
      __insertion_sort(first, first + chunk_size);
      first += chunk_size;
    }
    __insertion_sort(first, last);
  }

  template <class _RandomAccessIterator, class _Distance, class _Compare>
  void __chunk_insertion_sort (_RandomAccessIterator first, 
                               _RandomAccessIterator last,
                               _Distance chunk_size, _Compare _RWSTD_COMP)
  {
    while (last - first >= chunk_size)
    {
      __insertion_sort(first, first + chunk_size, _RWSTD_COMP);
      first += chunk_size;
    }
    __insertion_sort(first, last, _RWSTD_COMP);
  }

  template <class _RandomAccessIterator, class _Pointer, class _Distance, class _T>
  void __merge_sort_with_buffer (_RandomAccessIterator first, 
                                 _RandomAccessIterator last,
                                 _Pointer buffer, _Distance*, _T*)
  {
    _Distance len = last - first;
    _Pointer buffer_last = buffer + len;

    _Distance step_size = __stl_chunk_size;
    __chunk_insertion_sort(first, last, step_size);

    while (step_size < len)
    {
      __merge_sort_loop(first, last, buffer, step_size);
      step_size *= 2;
      __merge_sort_loop(buffer, buffer_last, first, step_size);
      step_size *= 2;
    }
  }

  template <class _RandomAccessIterator, class _Pointer, class _Distance, class _T,
  class _Compare>
  void __merge_sort_with_buffer (_RandomAccessIterator first, 
                                 _RandomAccessIterator last, _Pointer buffer,
                                 _Distance*, _T*, _Compare _RWSTD_COMP)
  {
    _Distance len = last - first;
    _Pointer buffer_last = buffer + len;

    _Distance step_size = __stl_chunk_size;
    __chunk_insertion_sort(first, last, step_size, _RWSTD_COMP);

    while (step_size < len)
    {
      __merge_sort_loop(first, last, buffer, step_size, _RWSTD_COMP);
      step_size *= 2;
      __merge_sort_loop(buffer, buffer_last, first, step_size, _RWSTD_COMP);
      step_size *= 2;
    }
  }

  template <class _RandomAccessIterator, class _Pointer, class _Distance, class _T>
  void __stable_sort_adaptive (_RandomAccessIterator first, 
                               _RandomAccessIterator last, _Pointer buffer,
                               _Distance buffer_size, _T*)
  {
    _Distance len = (last - first + 1) / 2;
    _RandomAccessIterator middle = first + len;
    if (len > buffer_size)
    {
      __stable_sort_adaptive(first, middle, buffer, buffer_size, 
                             _RWSTD_STATIC_CAST(_T*,0));
      __stable_sort_adaptive(middle, last, buffer, buffer_size, 
                             _RWSTD_STATIC_CAST(_T*,0));
    }
    else
    {
      __merge_sort_with_buffer(first, middle, buffer, 
                               _RWSTD_STATIC_CAST(_Distance*,0),
                               _RWSTD_STATIC_CAST(_T*,0));
      __merge_sort_with_buffer(middle, last, buffer, 
                               _RWSTD_STATIC_CAST(_Distance*,0),
                               _RWSTD_STATIC_CAST(_T*,0));
    }
    __merge_adaptive(first, middle, last, _Distance(middle - first), 
                     _Distance(last - middle), buffer, buffer_size, _RWSTD_STATIC_CAST(_T*,0));
  }

  template <class _RandomAccessIterator, class _Pointer, class _Distance, class _T,
  class _Compare>
  void __stable_sort_adaptive (_RandomAccessIterator first, 
                               _RandomAccessIterator last, _Pointer buffer,
                               _Distance buffer_size, _T*, _Compare _RWSTD_COMP)
  {
    _Distance len = (last - first + 1) / 2;
    _RandomAccessIterator middle = first + len;
    if (len > buffer_size)
    {
      __stable_sort_adaptive(first, middle, buffer, buffer_size,_RWSTD_STATIC_CAST(_T*,0),_RWSTD_COMP);
      __stable_sort_adaptive(middle, last, buffer, buffer_size, _RWSTD_STATIC_CAST(_T*,0),_RWSTD_COMP);
    }
    else
    {
      __merge_sort_with_buffer(first, middle, buffer, 
                               _RWSTD_STATIC_CAST(_Distance*,0),
                               _RWSTD_STATIC_CAST(_T*,0), _RWSTD_COMP);
      __merge_sort_with_buffer(middle, last, buffer, 
                               _RWSTD_STATIC_CAST(_Distance*,0),
                               _RWSTD_STATIC_CAST(_T*,0), _RWSTD_COMP);
    }
    __merge_adaptive(first, middle, last, _Distance(middle - first), 
                     _Distance(last-middle), buffer, buffer_size, _RWSTD_STATIC_CAST(_T*,0),_RWSTD_COMP);
  }

  template <class _RandomAccessIterator, class _T>
  void __partial_sort (_RandomAccessIterator first, _RandomAccessIterator middle,
                       _RandomAccessIterator last, _T*)
  {
    make_heap(first, middle);
    for (_RandomAccessIterator i = middle; i < last; ++i)
      if (*i < *first) 
        __pop_heap(first, middle, i, _T(*i), __distance_type(first));
    sort_heap(first, middle);
  }

  template <class _RandomAccessIterator, class _T, class _Compare>
  void __partial_sort (_RandomAccessIterator first, _RandomAccessIterator middle,
                       _RandomAccessIterator last, _T*, _Compare _RWSTD_COMP)
  {
    make_heap(first, middle, _RWSTD_COMP);
    for (_RandomAccessIterator i = middle; i < last; ++i)
      if (_RWSTD_COMP(*i,*first)) 
        __pop_heap(first, middle, i, _T(*i), _RWSTD_COMP, __distance_type(first));
    sort_heap(first, middle, _RWSTD_COMP);
  }

  template <class _InputIterator, class _RandomAccessIterator, class _Distance,
  class _T>
  _RandomAccessIterator __partial_sort_copy (_InputIterator first,
                                            _InputIterator last,
                                            _RandomAccessIterator result_first,
                                            _RandomAccessIterator result_last, 
                                            _Distance*, _T*)
  {
    if (result_first == result_last) return result_last;
    _RandomAccessIterator result_real_last = result_first;
    while(first != last && result_real_last != result_last)
      *result_real_last++ = *first++;
    make_heap(result_first, result_real_last);
    while (first != last)
    {
      if (*first < *result_first) 
        __adjust_heap(result_first, _Distance(0),
                      _Distance(result_real_last - result_first),
                      _T(*first));
      ++first;
    }
    sort_heap(result_first, result_real_last);
    return result_real_last;
  }

  template <class _InputIterator, class _RandomAccessIterator, class _Compare,
  class _Distance, class _T>
  _RandomAccessIterator __partial_sort_copy (_InputIterator first,
                                            _InputIterator last,
                                            _RandomAccessIterator result_first,
                                            _RandomAccessIterator result_last,
                                            _Compare _RWSTD_COMP, _Distance*, _T*)
  {
    if (result_first == result_last) return result_last;
    _RandomAccessIterator result_real_last = result_first;
    while(first != last && result_real_last != result_last)
      *result_real_last++ = *first++;
    make_heap(result_first, result_real_last, _RWSTD_COMP);
    while (first != last)
    {
      if (_RWSTD_COMP(*first,*result_first)) 
        __adjust_heap(result_first, _Distance(0),
                      _Distance(result_real_last - result_first), _T(*first),
                      _RWSTD_COMP);
      ++first;
    }
    sort_heap(result_first, result_real_last, _RWSTD_COMP);
    return result_real_last;
  }


// David R. Musser's Introspective Sorting algorithm
// (see www.cs.rpi.edu/~musser/gp/introsort.ps)
// O(N * log (N)) worst case complexity
template <class _RandomAccessIter, class __Distance, class _Compare>
void __introsort_loop (_RandomAccessIter __first, _RandomAccessIter __last,
                       __Distance __max_depth, _Compare __comp)
{
    for ( ; __stl_threshold < __last - __first; ) {
	if (0 == __max_depth) {
	    __partial_sort (__first, __last, __last,
		std::__value_type(__first), __comp);
	    break;
	}

        _RandomAccessIter __cut = __unguarded_partition (__first, __last, 
		__median (*__first, *(__first + (__last - __first) /2),
		*(__last - 1), __comp), __comp);

        // limit the depth of the recursion tree to log2 (last - first)
        // where first and last are the initial values passed in from sort()
        __introsort_loop (__cut, __last, __max_depth /= 2, __comp);
        __last = __cut;
    }
}


  template <class _RandomAccessIterator, class _T>
  void __nth_element (_RandomAccessIterator first, _RandomAccessIterator nth,
                      _RandomAccessIterator last, _T*)
  {
    while (last - first > 3)
    {
      _RandomAccessIterator cut = __unguarded_partition
      (first, last, _T(__median(*first, *(first + (last - first)/2),
                               *(last - 1))));
      if (cut <= nth)
        first = cut;
      else 
        last = cut;
    }
    __insertion_sort(first, last);
  }

  template <class _RandomAccessIterator, class _T, class _Compare>
  void __nth_element (_RandomAccessIterator first, _RandomAccessIterator nth,
                      _RandomAccessIterator last, _T*, _Compare _RWSTD_COMP)
  {
    while (last - first > 3)
    {
      _RandomAccessIterator cut = __unguarded_partition
      (first, last, _T(__median(*first, *(first + (last - first)/2), 
                               *(last - 1), _RWSTD_COMP)), _RWSTD_COMP);
      if (cut <= nth)
        first = cut;
      else 
        last = cut;
    }
    __insertion_sort(first, last, _RWSTD_COMP);
  }

//
// Binary search.
//

  template <class _ForwardIterator, class _T, class _Distance>
  _ForwardIterator __lower_bound (_ForwardIterator first, _ForwardIterator last,
                                 const _T& value, _Distance*,
                                 forward_iterator_tag)
  {
    _Distance len;
    __initialize(len, _Distance(0));
    distance(first, last, len);
    _Distance half;
    _ForwardIterator middle;

    while (len > 0)
    {
      half = len / 2;
      middle = first;
      advance(middle, half);
      if (*middle < value)
      {
        first = middle;
        ++first;
        len = len - half - 1;
      }
      else
        len = half;
    }
    return first;
  }

  template <class _RandomAccessIterator, class _T, class _Distance>
  _RandomAccessIterator __lower_bound (_RandomAccessIterator first,
                                      _RandomAccessIterator last, const _T& value,
                                      _Distance*, random_access_iterator_tag)
  {
    _Distance len = last - first;
    _Distance half;
    _RandomAccessIterator middle;

    while (len > 0)
    {
      half = len / 2;
      middle = first + half;
      if (*middle < value)
      {
        first = middle + 1;
        len = len - half - 1;
      }
      else
        len = half;
    }
    return first;
  }

  template <class _ForwardIterator, class _T, class _Compare, class _Distance>
  _ForwardIterator __lower_bound (_ForwardIterator first, _ForwardIterator last,
                                 const _T& value, _Compare _RWSTD_COMP, _Distance*,
                                 forward_iterator_tag)
  {
    _Distance len;
    __initialize(len, _Distance(0));
    distance(first, last, len);
    _Distance half;
    _ForwardIterator middle;

    while (len > 0)
    {
      half = len / 2;
      middle = first;
      advance(middle, half);
      if (_RWSTD_COMP(*middle, value))
      {
        first = middle;
        ++first;
        len = len - half - 1;
      }
      else
        len = half;
    }
    return first;
  }

  template <class _RandomAccessIterator, class _T, class _Compare, class _Distance>
  _RandomAccessIterator __lower_bound (_RandomAccessIterator first,
                                      _RandomAccessIterator last,
                                      const _T& value, 
                                      _Compare _RWSTD_COMP, 
                                      _Distance*,
                                      random_access_iterator_tag)
  {
    _Distance len = last - first;
    _Distance half;
    _RandomAccessIterator middle;

    while (len > 0)
    {
      half = len / 2;
      middle = first + half;
      if (_RWSTD_COMP(*middle, value))
      {
        first = middle + 1;
        len = len - half - 1;
      }
      else
        len = half;
    }
    return first;
  }

  template <class _ForwardIterator, class _T, class _Distance>
  _ForwardIterator __upper_bound (_ForwardIterator first, _ForwardIterator last,
                                 const _T& value, _Distance*,
                                 forward_iterator_tag)
  {
    _Distance len;
    __initialize(len, _Distance(0));
    distance(first, last, len);
    _Distance half;
    _ForwardIterator middle;

    while (len > 0)
    {
      half = len / 2;
      middle = first;
      advance(middle, half);
      if (value < *middle)
        len = half;
      else
      {
        first = middle;
        ++first;
        len = len - half - 1;
      }
    }
    return first;
  }

  template <class _RandomAccessIterator, class _T, class _Distance>
  _RandomAccessIterator __upper_bound (_RandomAccessIterator first,
                                      _RandomAccessIterator last, const _T& value,
                                      _Distance*, random_access_iterator_tag)
  {
    _Distance len = last - first;
    _Distance half;
    _RandomAccessIterator middle;

    while (len > 0)
    {
      half = len / 2;
      middle = first + half;
      if (value < *middle)
        len = half;
      else
      {
        first = middle + 1;
        len = len - half - 1;
      }
    }
    return first;
  }

  template <class _ForwardIterator, class _T, class _Compare, class _Distance>
  _ForwardIterator __upper_bound (_ForwardIterator first, _ForwardIterator last,
                                 const _T& value, _Compare _RWSTD_COMP, _Distance*,
                                 forward_iterator_tag)
  {
    _Distance len;
    __initialize(len, _Distance(0));
    distance(first, last, len);
    _Distance half;
    _ForwardIterator middle;

    while (len > 0)
    {
      half = len / 2;
      middle = first;
      advance(middle, half);
      if (_RWSTD_COMP(value, *middle))
        len = half;
      else {
        first = middle;
        ++first;
        len = len - half - 1;
      }
    }
    return first;
  }

  template <class _RandomAccessIterator, class _T, class _Compare, class _Distance>
  _RandomAccessIterator __upper_bound (_RandomAccessIterator first,
                                      _RandomAccessIterator last,
                                      const _T& value, 
                                      _Compare _RWSTD_COMP, _Distance*,
                                      random_access_iterator_tag)
  {
    _Distance len = last - first;
    _Distance half;
    _RandomAccessIterator middle;

    while (len > 0)
    {
      half = len / 2;
      middle = first + half;
      if (_RWSTD_COMP(value, *middle))
        len = half;
      else {
        first = middle + 1;
        len = len - half - 1;
      }
    }
    return first;
  }

  template <class _ForwardIterator, class _T, class _Distance>
  pair<_ForwardIterator, _ForwardIterator>
  __equal_range (_ForwardIterator first, _ForwardIterator last, const _T& value,
                 _Distance*, forward_iterator_tag)
  {
    _Distance len;
    __initialize(len, _Distance(0));
    distance(first, last, len);
    _Distance half;
    _ForwardIterator middle, left, right;

    while (len > 0)
    {
      half = len / 2;
      middle = first;
      advance(middle, half);
      if (*middle < value)
      {
        first = middle;
        ++first;
        len = len - half - 1;
      }
      else if (value < *middle)
        len = half;
      else
      {
        left = lower_bound(first, middle, value);
        advance(first, len);
        right = upper_bound(++middle, first, value);
        pair<_ForwardIterator, _ForwardIterator> tmp(left, right);
        return tmp;
      }
    }
    pair<_ForwardIterator, _ForwardIterator> tmp(first, first);
    return tmp;
  }

  template <class _RandomAccessIterator, class _T, class _Distance>
  pair<_RandomAccessIterator, _RandomAccessIterator>
  __equal_range (_RandomAccessIterator first, _RandomAccessIterator last,
                 const _T& value, _Distance*, random_access_iterator_tag)
  {
    _Distance len = last - first;
    _Distance half;
    _RandomAccessIterator middle, left, right;

    while (len > 0)
    {
      half = len / 2;
      middle = first + half;
      if (*middle < value)
      {
        first = middle + 1;
        len = len - half - 1;
      }
      else if (value < *middle)
        len = half;
      else
      {
        left = lower_bound(first, middle, value);
        right = upper_bound(++middle, first + len, value);
        pair<_RandomAccessIterator,_RandomAccessIterator> tmp(left,right);
        return tmp;
      }
    }
    pair<_RandomAccessIterator, _RandomAccessIterator> tmp(first, first);
    return tmp;
  }

  template <class _ForwardIterator, class _T, class _Compare, class _Distance>
  pair<_ForwardIterator, _ForwardIterator>
  __equal_range (_ForwardIterator first, _ForwardIterator last, const _T& value,
                 _Compare _RWSTD_COMP, _Distance*, forward_iterator_tag)
  {
    _Distance len;
    __initialize(len, _Distance(0));
    distance(first, last, len);
    _Distance half;
    _ForwardIterator middle, left, right;

    while (len > 0)
    {
      half = len / 2;
      middle = first;
      advance(middle, half);
      if (_RWSTD_COMP(*middle, value))
      {
        first = middle;
        ++first;
        len = len - half - 1;
      }
      else if (_RWSTD_COMP(value, *middle))
        len = half;
      else
      {
        left = lower_bound(first, middle, value, _RWSTD_COMP);
        advance(first, len);
        right = upper_bound(++middle, first, value, _RWSTD_COMP);
        pair<_ForwardIterator, _ForwardIterator> tmp(left, right);
        return tmp;
      }
    }
    pair<_ForwardIterator, _ForwardIterator> tmp(first, first);
    return tmp;
  }           

  template <class _RandomAccessIterator, class _T, class _Compare, class _Distance>
  pair<_RandomAccessIterator, _RandomAccessIterator>
  __equal_range (_RandomAccessIterator first, _RandomAccessIterator last,
                 const _T& value, _Compare _RWSTD_COMP, _Distance*,
                 random_access_iterator_tag)
  {
    _Distance len = last - first;
    _Distance half;
    _RandomAccessIterator middle, left, right;

    while (len > 0)
    {
      half = len / 2;
      middle = first + half;
      if (_RWSTD_COMP(*middle, value))
      {
        first = middle + 1;
        len = len - half - 1;
      }
      else if (_RWSTD_COMP(value, *middle))
        len = half;
      else
      {
        left = lower_bound(first, middle, value, _RWSTD_COMP);
        right = upper_bound(++middle, first + len, value, _RWSTD_COMP);
        pair<_RandomAccessIterator,_RandomAccessIterator> tmp(left, right);
        return tmp;
      }
    }
    pair<_RandomAccessIterator, _RandomAccessIterator> tmp(first, first);
    return tmp;
  }           

//
// Merge
//

  template <class _InputIterator1, class _InputIterator2, class _OutputIterator>
  _OutputIterator merge (_InputIterator1 first1, _InputIterator1 last1,
                        _InputIterator2 first2, _InputIterator2 last2,
                        _OutputIterator result)
  {
    while (first1 != last1 && first2 != last2)
    {
      if (*first2 < *first1)
        *result++ = *first2++;
      else
        *result++ = *first1++;
    }
    return copy(first2, last2, copy(first1, last1, result));
  }

  template <class _InputIterator1, class _InputIterator2, class _OutputIterator,
  class _Compare>
  _OutputIterator merge (_InputIterator1 first1, _InputIterator1 last1,
                        _InputIterator2 first2, _InputIterator2 last2,
                        _OutputIterator result, _Compare _RWSTD_COMP)
  {
    while (first1 != last1 && first2 != last2)
    {
      if (_RWSTD_COMP(*first2, *first1))
        *result++ = *first2++;
      else
        *result++ = *first1++;
    }
    return copy(first2, last2, copy(first1, last1, result));
  }

  template <class _BidirectionalIterator, class _Distance>
  void __merge_without_buffer (_BidirectionalIterator first,
                               _BidirectionalIterator middle,
                               _BidirectionalIterator last,
                               _Distance len1, _Distance len2)
  {
    if (len1 == 0 || len2 == 0) return;
    if (len1 + len2 == 2)
    {
      if (*middle < *first) iter_swap(first, middle);
      return;
    }
    _BidirectionalIterator first_cut = first;
    _BidirectionalIterator second_cut = middle;
    _Distance len11;
    __initialize(len11, _Distance(0));
    _Distance len22;
    __initialize(len22, _Distance(0));
    if (len1 > len2)
    {
      len11 = len1 / 2;
      advance(first_cut, len11);
      second_cut = lower_bound(middle, last, *first_cut);
      distance(middle, second_cut, len22);
    }
    else
    {
      len22 = len2 / 2;
      advance(second_cut, len22);
      first_cut = upper_bound(first, middle, *second_cut);
      distance(first, first_cut, len11);
    }
    rotate(first_cut, middle, second_cut);
    _BidirectionalIterator new_middle = first_cut;
    advance(new_middle, len22);
    __merge_without_buffer(first, first_cut, new_middle, len11, len22);
    __merge_without_buffer(new_middle, second_cut, last, len1 - len11,
                           len2 - len22);
  }

  template <class _BidirectionalIterator, class _Distance, class _Compare>
  void __merge_without_buffer (_BidirectionalIterator first,
                               _BidirectionalIterator middle,
                               _BidirectionalIterator last,
                               _Distance len1, _Distance len2, _Compare _RWSTD_COMP)
  {
    if (len1 == 0 || len2 == 0) return;
    if (len1 + len2 == 2)
    {
      if (_RWSTD_COMP(*middle, *first)) iter_swap(first, middle);
      return;
    }
    _BidirectionalIterator first_cut = first;
    _BidirectionalIterator second_cut = middle;
    _Distance len11;
    __initialize(len11, _Distance(0));
    _Distance len22;
    __initialize(len22, _Distance(0));
    if (len1 > len2)
    {
      len11 = len1 / 2;
      advance(first_cut, len11);
      second_cut = lower_bound(middle, last, *first_cut, _RWSTD_COMP);
      distance(middle, second_cut, len22);
    }
    else
    {
      len22 = len2 / 2;
      advance(second_cut, len22);
      first_cut = upper_bound(first, middle, *second_cut, _RWSTD_COMP);
      distance(first, first_cut, len11);
    }
    rotate(first_cut, middle, second_cut);
    _BidirectionalIterator new_middle = first_cut;
    advance(new_middle, len22);
    __merge_without_buffer(first, first_cut, new_middle, len11, len22, _RWSTD_COMP);
    __merge_without_buffer(new_middle, second_cut, last, len1 - len11,
                           len2 - len22, _RWSTD_COMP);
  }

  template <class _BidirectionalIterator1, class _BidirectionalIterator2,
  class _Distance>
  _BidirectionalIterator1 __rotate_adaptive (_BidirectionalIterator1 first,
                                            _BidirectionalIterator1 middle,
                                            _BidirectionalIterator1 last,
                                            _Distance len1, _Distance len2,
                                            _BidirectionalIterator2 buffer,
                                            _Distance buffer_size)
  {
    _BidirectionalIterator2 buffer_end;
    if (len1 > len2 && len2 <= buffer_size)
    {
      buffer_end = copy(middle, last, buffer);
      copy_backward(first, middle, last);
      return copy(buffer, buffer_end, first);
    }
    else if (len1 <= buffer_size)
    {
      buffer_end = copy(first, middle, buffer);
      copy(middle, last, first);
      return copy_backward(buffer, buffer_end, last);
    }
    else
    {
      rotate(first, middle, last);
      advance(first, len2);
      return first;
    }
  }

  template <class _BidirectionalIterator1, class _BidirectionalIterator2,
  class _BidirectionalIterator3>
  _BidirectionalIterator3 __merge_backward (_BidirectionalIterator1 first1,
                                           _BidirectionalIterator1 last1,
                                           _BidirectionalIterator2 first2,
                                           _BidirectionalIterator2 last2,
                                           _BidirectionalIterator3 result)
  {
    if (first1 == last1) return copy_backward(first2, last2, result);
    if (first2 == last2) return copy_backward(first1, last1, result);
    --last1;
    --last2;
    while (true)
    {
      if (*last2 < *last1)
      {
        *--result = *last1;
        if (first1 == last1) return copy_backward(first2, ++last2, result);
        --last1;
      }
      else
      {
        *--result = *last2;
        if (first2 == last2) return copy_backward(first1, ++last1, result);
        --last2;
      }
    }
  }

  template <class _BidirectionalIterator1, class _BidirectionalIterator2,
  class _BidirectionalIterator3, class _Compare>
  _BidirectionalIterator3 __merge_backward (_BidirectionalIterator1 first1,
                                           _BidirectionalIterator1 last1,
                                           _BidirectionalIterator2 first2,
                                           _BidirectionalIterator2 last2,
                                           _BidirectionalIterator3 result,
                                           _Compare _RWSTD_COMP)
  {
    if (first1 == last1) return copy_backward(first2, last2, result);
    if (first2 == last2) return copy_backward(first1, last1, result);
    --last1;
    --last2;
    while (true)
    {
      if (_RWSTD_COMP(*last2, *last1))
      {
        *--result = *last1;
        if (first1 == last1) return copy_backward(first2, ++last2, result);
        --last1;
      }
      else
      {
        *--result = *last2;
        if (first2 == last2) return copy_backward(first1, ++last1, result);
        --last2;
      }
    }
  }

  template <class _BidirectionalIterator, class _Distance, class _Pointer, class _T>
  void __merge_adaptive (_BidirectionalIterator first,
                         _BidirectionalIterator middle,
                         _BidirectionalIterator last, _Distance len1,_Distance len2,
                         _Pointer buffer, _Distance buffer_size, _T*)
  {
    if (len1 <= len2 && len1 <= buffer_size)
    {
      _Pointer end_buffer = copy(first, middle, buffer);
      merge(buffer, end_buffer, middle, last, first);
    }
    else if (len2 <= buffer_size)
    {
      _Pointer end_buffer = copy(middle, last, buffer);
      __merge_backward(first, middle, buffer, end_buffer, last);
    }
    else
    {
      _BidirectionalIterator first_cut = first;
      _BidirectionalIterator second_cut = middle;
      _Distance len11;
      __initialize(len11, _Distance(0));
      _Distance len22;
      __initialize(len22, _Distance(0));
      if (len1 > len2)
      {
        len11 = len1 / 2;
        advance(first_cut, len11);
        second_cut = lower_bound(middle, last, *first_cut);
        distance(middle, second_cut, len22);   
      }
      else
      {
        len22 = len2 / 2;
        advance(second_cut, len22);
        first_cut = upper_bound(first, middle, *second_cut);
        distance(first, first_cut, len11);
      }
      _BidirectionalIterator new_middle =
      __rotate_adaptive(first_cut, middle, second_cut, len1 - len11,
                        len22, buffer, buffer_size);
      __merge_adaptive(first, first_cut, new_middle, len11, len22, buffer,
                       buffer_size, _RWSTD_STATIC_CAST(_T*,0));
      __merge_adaptive(new_middle, second_cut, last, len1 - len11,
                       len2 - len22, buffer, buffer_size, _RWSTD_STATIC_CAST(_T*,0));
    }
  }

  template <class _BidirectionalIterator, class _Distance, class _Pointer, class _T,
  class _Compare>
  void __merge_adaptive (_BidirectionalIterator first,
                         _BidirectionalIterator middle,
                         _BidirectionalIterator last, _Distance len1,_Distance len2,
                         _Pointer buffer, _Distance buffer_size, _T*, _Compare _RWSTD_COMP)
  {
    if (len1 <= len2 && len1 <= buffer_size)
    {
      _Pointer end_buffer = copy(first, middle, buffer);
      merge(buffer, end_buffer, middle, last, first, _RWSTD_COMP);
    }
    else if (len2 <= buffer_size)
    {
      _Pointer end_buffer = copy(middle, last, buffer);
      __merge_backward(first, middle, buffer, end_buffer, last, _RWSTD_COMP);
    }
    else
    {
      _BidirectionalIterator first_cut = first;
      _BidirectionalIterator second_cut = middle;
      _Distance len11;
      __initialize(len11, _Distance(0));
      _Distance len22;
      __initialize(len22, _Distance(0));
      if (len1 > len2)
      {
        len11 = len1 / 2;
        advance(first_cut, len11);
        second_cut = lower_bound(middle, last, *first_cut, _RWSTD_COMP);
        distance(middle, second_cut, len22);   
      }
      else
      {
        len22 = len2 / 2;
        advance(second_cut, len22);
        first_cut = upper_bound(first, middle, *second_cut, _RWSTD_COMP);
        distance(first, first_cut, len11);
      }
      _BidirectionalIterator new_middle =
        __rotate_adaptive(first_cut, middle, second_cut, len1 - len11,
                          len22, buffer, buffer_size);
      __merge_adaptive(first, first_cut, new_middle, len11, len22, buffer,
                       buffer_size, _RWSTD_STATIC_CAST(_T*,0), _RWSTD_COMP);
      __merge_adaptive(new_middle, second_cut, last, len1 - len11,
                       len2 - len22, buffer, buffer_size, _RWSTD_STATIC_CAST(_T*,0), _RWSTD_COMP);
    }
  }

  template <class _BidirectionalIterator, class _Distance, class _Pointer, class _T>
  void __inplace_merge (_BidirectionalIterator first,
                        _BidirectionalIterator middle, 
                        _BidirectionalIterator last, _Distance len1,
                        _Distance len2, pair<_Pointer, _Distance> p, _T*)
  {
    if (p.first == 0)
      __merge_without_buffer(first, middle, last, len1, len2);
    else
    {
      _Distance len = (min)(p.second, len1 + len2);
      fill_n(raw_storage_iterator<_Pointer, _T>(p.first), len, *first);
      __merge_adaptive(first, middle, last, len1, len2, p.first,
                       p.second, _RWSTD_STATIC_CAST(_T*,0));
      __RWSTD::__destroy(p.first, p.first + len);
      return_temporary_buffer(p.first);
    }
  }

  template <class _BidirectionalIterator, class _Distance, class _Pointer, class _T,
  class _Compare>
  void __inplace_merge (_BidirectionalIterator first,
                        _BidirectionalIterator middle,
                        _BidirectionalIterator last, _Distance len1,
                        _Distance len2, pair<_Pointer, _Distance> p, _T*,
                        _Compare _RWSTD_COMP)
  {
    if (p.first == 0)
      __merge_without_buffer(first, middle, last, len1, len2, _RWSTD_COMP);
    else
    {
      _Distance len = (min)(p.second, len1 + len2);
      fill_n(raw_storage_iterator<_Pointer, _T>(p.first), len, *first);
      __merge_adaptive(first, middle, last, len1, len2, p.first,
                       p.second, _RWSTD_STATIC_CAST(_T*,0), _RWSTD_COMP); 
      __RWSTD::__destroy(p.first, p.first + len);
      return_temporary_buffer(p.first);
    }
  }

//
// Set operations.
//

  template <class _InputIterator1, class _InputIterator2>
  bool includes (_InputIterator1 first1, _InputIterator1 last1,
                 _InputIterator2 first2, _InputIterator2 last2)
  {
    while (first1 != last1 && first2 != last2)
    {
      if (*first2 < *first1)
        return false;
      else if(*first1 < *first2) 
        ++first1;
      else
        ++first1, ++first2;
    }
    return first2 == last2;
  }

  template <class _InputIterator1, class _InputIterator2, class _Compare>
  bool includes (_InputIterator1 first1, _InputIterator1 last1,
                 _InputIterator2 first2, _InputIterator2 last2, _Compare _RWSTD_COMP)
  {
    while (first1 != last1 && first2 != last2)
    {
      if (_RWSTD_COMP(*first2, *first1))
        return false;
      else if(_RWSTD_COMP(*first1, *first2)) 
        ++first1;
      else
        ++first1, ++first2;
    }
    return first2 == last2;
  }

  template <class _InputIterator1, class _InputIterator2, class _OutputIterator>
  _OutputIterator set_union (_InputIterator1 first1, _InputIterator1 last1,
                            _InputIterator2 first2, _InputIterator2 last2,
                            _OutputIterator result)
  {
    while (first1 != last1 && first2 != last2)
    {
      if (*first1 < *first2)
        *result++ = *first1++;
      else if (*first2 < *first1)
        *result++ = *first2++;
      else
      {
        *result++ = *first1++;
        first2++;
      }
    }
    return copy(first2, last2, copy(first1, last1, result));
  }

  template <class _InputIterator1, class _InputIterator2, class _OutputIterator,
  class _Compare>
  _OutputIterator set_union (_InputIterator1 first1, _InputIterator1 last1,
                            _InputIterator2 first2, _InputIterator2 last2,
                            _OutputIterator result, _Compare _RWSTD_COMP)
  {
    while (first1 != last1 && first2 != last2)
    {
      if (_RWSTD_COMP(*first1, *first2))
        *result++ = *first1++;
      else if (_RWSTD_COMP(*first2, *first1))
        *result++ = *first2++;
      else
      {
        *result++ = *first1++;
        ++first2;
      }
    }
    return copy(first2, last2, copy(first1, last1, result));
  }

  template <class _InputIterator1, class _InputIterator2, class _OutputIterator>
  _OutputIterator set_intersection (_InputIterator1 first1, _InputIterator1 last1,
                                   _InputIterator2 first2, _InputIterator2 last2,
                                   _OutputIterator result)
  {
    while (first1 != last1 && first2 != last2)
    {
      if (*first1 < *first2)
        ++first1;
      else if (*first2 < *first1)
        ++first2;
      else
      {
        *result++ = *first1++;
        ++first2;
      }
    }
    return result;
  }

  template <class _InputIterator1, class _InputIterator2, class _OutputIterator,
  class _Compare>
  _OutputIterator set_intersection (_InputIterator1 first1, _InputIterator1 last1,
                                   _InputIterator2 first2, _InputIterator2 last2,
                                   _OutputIterator result, _Compare _RWSTD_COMP)
  {
    while (first1 != last1 && first2 != last2)
    {
      if (_RWSTD_COMP(*first1, *first2))
        ++first1;
      else if (_RWSTD_COMP(*first2, *first1))
        ++first2;
      else
      {
        *result++ = *first1++;
        ++first2;
      }
    }
    return result;
  }

  template <class _InputIterator1, class _InputIterator2, class _OutputIterator>
  _OutputIterator set_difference (_InputIterator1 first1, _InputIterator1 last1,
                                 _InputIterator2 first2, _InputIterator2 last2,
                                 _OutputIterator result)
  {
    while (first1 != last1 && first2 != last2)
    {
      if (*first1 < *first2)
        *result++ = *first1++;
      else if (*first2 < *first1)
        ++first2;
      else
      {
        ++first1;
        ++first2;
      }
    }
    return copy(first1, last1, result);
  }

  template <class _InputIterator1, class _InputIterator2, class _OutputIterator, 
  class _Compare>
  _OutputIterator set_difference (_InputIterator1 first1, _InputIterator1 last1,
                                 _InputIterator2 first2, _InputIterator2 last2, 
                                 _OutputIterator result, _Compare _RWSTD_COMP)
  {
    while (first1 != last1 && first2 != last2)
    {
      if (_RWSTD_COMP(*first1, *first2))
        *result++ = *first1++;
      else if (_RWSTD_COMP(*first2, *first1))
        ++first2;
      else
      {
        ++first1;
        ++first2;
      }
    }
    return copy(first1, last1, result);
  }

  template <class _InputIterator1, class _InputIterator2, class _OutputIterator>
  _OutputIterator set_symmetric_difference (_InputIterator1 first1,
                                           _InputIterator1 last1,
                                           _InputIterator2 first2,
                                           _InputIterator2 last2,
                                           _OutputIterator result)
  {
    while (first1 != last1 && first2 != last2)
    {
      if (*first1 < *first2)
        *result++ = *first1++;
      else if (*first2 < *first1)
        *result++ = *first2++;
      else
      {
        ++first1;
        ++first2;
      }
    }
    return copy(first2, last2, copy(first1, last1, result));
  }

  template <class _InputIterator1, class _InputIterator2, class _OutputIterator,
  class _Compare>
  _OutputIterator set_symmetric_difference (_InputIterator1 first1,
                                           _InputIterator1 last1,
                                           _InputIterator2 first2,
                                           _InputIterator2 last2,
                                           _OutputIterator result, _Compare _RWSTD_COMP)
  {
    while (first1 != last1 && first2 != last2)
    {
      if (_RWSTD_COMP(*first1, *first2))
        *result++ = *first1++;
      else if (_RWSTD_COMP(*first2, *first1))
        *result++ = *first2++;
      else
      {
        ++first1;
        ++first2;
      }
    }
    return copy(first2, last2, copy(first1, last1, result));
  }

//
// Heap operations.
//

  template <class _RandomAccessIterator, class _Distance, class _T>
  void __push_heap (_RandomAccessIterator first, _Distance holeIndex,
                    _Distance topIndex, _T value)
  {
    _Distance parent = (holeIndex - 1) / 2;
    while (holeIndex > topIndex && *(first + parent) < value)
    {
      *(first + holeIndex) = *(first + parent);
      holeIndex = parent;
      parent = (holeIndex - 1) / 2;
    }    
    *(first + holeIndex) = value;
  }

  template <class _RandomAccessIterator, class _Distance, class _T, class _Compare>
  void __push_heap (_RandomAccessIterator first, _Distance holeIndex,
                    _Distance topIndex, _T value, _Compare _RWSTD_COMP)
  {
    _Distance parent = (holeIndex - 1) / 2;
    while (holeIndex > topIndex && _RWSTD_COMP(*(first + parent), value))
    {
      *(first + holeIndex) = *(first + parent);
      holeIndex = parent;
      parent = (holeIndex - 1) / 2;
    }
    *(first + holeIndex) = value;
  }

  template <class _RandomAccessIterator, class _Distance, class _T>
  void __adjust_heap (_RandomAccessIterator first, _Distance holeIndex,
                      _Distance len, _T value)
  {
    _Distance topIndex = holeIndex;
    _Distance secondChild = 2 * holeIndex + 2;
    while (secondChild < len)
    {
      if (*(first + secondChild) < *(first + (secondChild - 1)))
        secondChild--;
      *(first + holeIndex) = *(first + secondChild);
      holeIndex = secondChild;
      secondChild = 2 * (secondChild + 1);
    }
    if (secondChild == len)
    {
      *(first + holeIndex) = *(first + (secondChild - 1));
      holeIndex = secondChild - 1;
    }
    __push_heap(first, holeIndex, topIndex, value);
  }

  template <class _RandomAccessIterator, class _Distance, class _T, class _Compare>
  void __adjust_heap (_RandomAccessIterator first, _Distance holeIndex,
                      _Distance len, _T value, _Compare _RWSTD_COMP)
  {
    _Distance topIndex = holeIndex;
    _Distance secondChild = 2 * holeIndex + 2;
    while (secondChild < len)
    {
      if (_RWSTD_COMP(*(first + secondChild), *(first + (secondChild - 1))))
        secondChild--;
      *(first + holeIndex) = *(first + secondChild);
      holeIndex = secondChild;
      secondChild = 2 * (secondChild + 1);
    }
    if (secondChild == len)
    {
      *(first + holeIndex) = *(first + (secondChild - 1));
      holeIndex = secondChild - 1;
    }
    __push_heap(first, holeIndex, topIndex, value, _RWSTD_COMP);
  }

  template <class _RandomAccessIterator, class _T, class _Distance>
  void __make_heap (_RandomAccessIterator first, _RandomAccessIterator last, _T*,
                    _Distance*)
  {
    _Distance len = last - first;
    _Distance parent = (len - 2)/2;
    while (true)
    {
      __adjust_heap(first, parent, len, _T(*(first + parent)));
      if (parent == 0) return;
      parent--;
    }
  }

  template <class _RandomAccessIterator, class _Compare, class _T, class _Distance>
  void __make_heap (_RandomAccessIterator first, _RandomAccessIterator last,
                    _Compare _RWSTD_COMP, _T*, _Distance*)
  {
    _Distance len = last - first;
    _Distance parent = (len - 2)/2;
    while (true)
    {
      __adjust_heap(first, parent, len, _T(*(first + parent)), _RWSTD_COMP);
      if (parent == 0)
        return;
      parent--;
    }
  }

  template <class _RandomAccessIterator>
  void sort_heap (_RandomAccessIterator first, _RandomAccessIterator last)
  {
    while (last - first > 1) pop_heap(first, last--);
  }

  template <class _RandomAccessIterator, class _Compare>
  void sort_heap (_RandomAccessIterator first, _RandomAccessIterator last,
                  _Compare _RWSTD_COMP)
  {
    while (last - first > 1) pop_heap(first, last--, _RWSTD_COMP);
  }

//
// Minimum and maximum.
//

  template <class _ForwardIterator>
  _ForwardIterator min_element (_ForwardIterator first, _ForwardIterator last)
  {
    if (first == last) return first;
    _ForwardIterator result = first;
    while (++first != last) 
      if (*first < *result) result = first;
    return result;
  }

  template <class _ForwardIterator, class _Compare>
  _ForwardIterator min_element (_ForwardIterator first, _ForwardIterator last,
                               _Compare _RWSTD_COMP)
  {
    if (first == last) return first;
    _ForwardIterator result = first;
    while (++first != last) 
      if (_RWSTD_COMP(*first, *result)) result = first;
    return result;
  }

  template <class _ForwardIterator>
  _ForwardIterator max_element (_ForwardIterator first, _ForwardIterator last)
  {
    if (first == last) return first;
    _ForwardIterator result = first;
    while (++first != last) 
      if (*result < *first) result = first;
    return result;
  }

  template <class _ForwardIterator, class _Compare>
  _ForwardIterator max_element (_ForwardIterator first, _ForwardIterator last,
                               _Compare _RWSTD_COMP)
  {
    if (first == last) return first;
    _ForwardIterator result = first;
    while (++first != last) 
      if (_RWSTD_COMP(*result, *first)) result = first;
    return result;
  }

  template <class _InputIterator1, class _InputIterator2>
  bool lexicographical_compare (_InputIterator1 first1, _InputIterator1 last1,
                                _InputIterator2 first2, _InputIterator2 last2)
  {
    while (first1 != last1 && first2 != last2)
    {
      if (*first1 < *first2)     return true;
      if (*first2++ < *first1++) return false;
    }
    return first1 == last1 && first2 != last2;
  }

  template <class _InputIterator1, class _InputIterator2, class _Compare>
  bool lexicographical_compare(_InputIterator1 first1, _InputIterator1 last1,
                               _InputIterator2 first2,_InputIterator2 last2,
                               _Compare _RWSTD_COMP)
  {
    while (first1 != last1 && first2 != last2)
    {
      if (_RWSTD_COMP(*first1, *first2))     return true;
      if (_RWSTD_COMP(*first2++, *first1++)) return false;
    }
    return first1 == last1 && first2 != last2;
  }

//
// Permutations.
//

  template <class _BidirectionalIterator>
  bool next_permutation (_BidirectionalIterator first,
                         _BidirectionalIterator last)
  {
    if (first == last) return false;
    _BidirectionalIterator i = first;
    ++i;
    if (i == last) return false;
    i = last;
    --i;

    for (;;)
    {
      _BidirectionalIterator ii = i--;
      if (*i < *ii)
      {
        _BidirectionalIterator j = last;
        while (!(*i < *--j))
          ;
        iter_swap(i, j);
        reverse(ii, last);
        return true;
      }
      if (i == first)
      {
        reverse(first, last);
        return false;
      }
    }
  }

  template <class _BidirectionalIterator, class _Compare>
  bool next_permutation (_BidirectionalIterator first, _BidirectionalIterator last,
                         _Compare _RWSTD_COMP)
  {
    if (first == last) return false;
    _BidirectionalIterator i = first;
    ++i;
    if (i == last) return false;
    i = last;
    --i;

    for (;;)
    {
      _BidirectionalIterator ii = i--;
      if (_RWSTD_COMP(*i, *ii))
      {
        _BidirectionalIterator j = last;
        while (!_RWSTD_COMP(*i, *--j))
          ;
        iter_swap(i, j);
        reverse(ii, last);
        return true;
      }
      if (i == first)
      {
        reverse(first, last);
        return false;
      }
    }
  }

  template <class _BidirectionalIterator>
  bool prev_permutation (_BidirectionalIterator first,
                         _BidirectionalIterator last)
  {
    if (first == last) return false;
    _BidirectionalIterator i = first;
    ++i;
    if (i == last) return false;
    i = last;
    --i;

    for (;;)
    {
      _BidirectionalIterator ii = i--;
      if (*ii < *i)
      {
        _BidirectionalIterator j = last;
        while (!(*--j < *i))
          ;
        iter_swap(i, j);
        reverse(ii, last);
        return true;
      }
      if (i == first)
      {
        reverse(first, last);
        return false;
      }
    }
  }

  template <class _BidirectionalIterator, class _Compare>
  bool prev_permutation (_BidirectionalIterator first, _BidirectionalIterator last,
                         _Compare _RWSTD_COMP)
  {
    if (first == last) return false;
    _BidirectionalIterator i = first;
    ++i;
    if (i == last) return false;
    i = last;
    --i;

    for(;;)
    {
      _BidirectionalIterator ii = i--;
      if (_RWSTD_COMP(*ii, *i))
      {
        _BidirectionalIterator j = last;
        while (!_RWSTD_COMP(*--j, *i))
          ;
        iter_swap(i, j);
        reverse(ii, last);
        return true;
      }
      if (i == first)
      {
        reverse(first, last);
        return false;
      }
    }
  }

#ifndef _RWSTD_NO_NAMESPACE
}
#endif

#pragma enable_warn

