/***************************************************************************
 *
 * ostream.cc - Definitions for the Standard Library ostream classes
 *
 ***************************************************************************
 *
 * Copyright (c) 1994-1999 Rogue Wave Software, Inc.  All Rights Reserved.
 *
 * This computer software is owned by Rogue Wave Software, Inc. and is
 * protected by U.S. copyright laws and other laws and by international
 * treaties.  This computer software is furnished by Rogue Wave Software,
 * Inc. pursuant to a written license agreement and may be used, copied,
 * transmitted, and stored only in accordance with the terms of such
 * license and with the inclusion of the above copyright notice.  This
 * computer software or any other copies thereof may not be provided or
 * otherwise made available to any other person.
 *
 * U.S. Government Restricted Rights.  This computer software is provided
 * with Restricted Rights.  Use, duplication, or disclosure by the
 * Government is subject to restrictions as set forth in subparagraph (c)
 * (1) (ii) of The Rights in Technical Data and Computer Software clause
 * at DFARS 252.227-7013 or subparagraphs (c) (1) and (2) of the
 * Commercial Computer Software  Restricted Rights at 48 CFR 52.227-19,
 * as applicable.  Manufacturer is Rogue Wave Software, Inc., 5500
 * Flatiron Parkway, Boulder, Colorado 80301 USA.
 *
 **************************************************************************/

#include <streambuf>

#ifndef _RWSTD_NO_NAMESPACE
namespace std {
#endif

  /*
   * basic_ostream(basic_streambuf *)
   */

  template<class _charT, class _Traits> 
  basic_ostream<_charT, _Traits>::basic_ostream(basic_streambuf<_charT, _Traits> *sb)
  {
    if ( sb )
    {
      if ( sb->which_open_mode() & ios_base::out ) 
        this->init(sb);
      else
        this->init(0);
    } 
    else
      this->init(0);
  }

  /*
   * basic_ostream( )
   */

  template<class _charT, class _Traits>
  basic_ostream<_charT, _Traits>::basic_ostream( )
  {
  }

  /*
   * ~basic_ostream()
   */

  template<class _charT, class _Traits>
  basic_ostream<_charT, _Traits>::~basic_ostream()
  {

  }

  /*
   * basic_ostream& flush()  
   */
 
  template<class _charT, class _Traits>
  basic_ostream<_charT, _Traits>&
  basic_ostream<_charT, _Traits>::flush()
  {
  
    if(this->rdbuf())
    {
#ifdef _RWSTD_MULTI_THREAD
      _RWSTDGuard guard(this->rdbuf()->buffer_mutex_);
#endif
      if(this->rdbuf()->pubsync() == -1)
        this->setstate(ios_base::badbit);
    }

    return *this;
  }
  
  /*
   * basic_ostream& operator<<(basic_ostream& (*pf)(basic_ostream&))
   *
   * these are the ostream manipulators (endl, ends, flush)
   */
  template<class _charT, class _Traits>
  basic_ostream<_charT, _Traits>&
  basic_ostream<_charT, _Traits>::
  operator<<(basic_ostream<_charT, _Traits>& (*pf)(basic_ostream<_charT, _Traits>& ))
  {
    return (*pf)(*this);
  }

  /*
   * basic_ostream& operator<<(ios_base& (*pf)(ios_base&))
   *
   * outputs the ios_base manipulators
   */

  template<class _charT, class _Traits>
  basic_ostream<_charT, _Traits>&
  basic_ostream<_charT, _Traits>::
  operator<<(ios_base& (*pf)(ios_base&))
  {
    (*pf)(*this);
    return *this;
  }

  /*
   * basic_ostream& operator<<(basic_ios& (*pf)(basic_ios& ))
   *
   * these are the ios manipulators (dec, hex, ...)
   */
  template<class _charT, class _Traits>
  basic_ostream<_charT, _Traits>&
  basic_ostream<_charT, _Traits>::
  operator<<(basic_ios<_charT, _Traits>& (*pf)(basic_ios<_charT, _Traits>&))
  {
    (*pf)(*this);
    return *this;
  }

  /*
   * basic_ostream& operator<< (basic_ostream<_charT, _Traits>& os, const _charT *)
   *
   */

  template<class _charT, class _Traits>
  basic_ostream<_charT, _Traits>&
  _RWSTDExportTemplate operator<< ( basic_ostream<_charT, _Traits>& os, const _charT *s)
  {
    ios_base::iostate err = 0;

#ifndef _RWSTD_NO_EXCEPTIONS
    try {
#endif
  
      if ( s )
      {   
        _TYPENAME basic_ostream<_charT, _Traits>::sentry opfx(os);
        if (opfx)
        {
          int   dlen = (int)_Traits::length(s);
          int   pad = (int)os.width() - dlen;

          // place right padding
          if( !(os.flags() & ios_base::adjustfield) ||  (os.flags() & ios_base::right) )
          {
            while(--pad >= 0)
            {
              if( _Traits::eq_int_type(os.rdbuf()->sputc(os.fill()),_Traits::eof())) {
                err = ios_base::badbit;
                break;
              }
            }
          }
         
          // output internal padding
          if(os.good() && (os.flags() & ios_base::internal))
          {
            while(--pad >= 0)
            {
              if( _Traits::eq_int_type(os.rdbuf()->sputc(os.fill()),_Traits::eof()))
              {
                err = ios_base::badbit;
                break;
              }
            }
          }

          if(os.good() && dlen) {
            if(os.rdbuf() && (os.rdbuf()->sputn(s, dlen) != dlen))
              err = ios_base::badbit;
          }

          // output left padding. 
          if(os.good() && (os.flags() & ios_base::left))
          {
            while(--pad >= 0)
            {
              if( _Traits::eq_int_type(os.rdbuf()->sputc(os.fill()),_Traits::eof()))
              {
                err = ios_base::badbit;
                break;
              }
            }
          }

          os.width(0);
        }
      }
      else
        err = ios_base::badbit;

#ifndef _RWSTD_NO_EXCEPTIONS
    }
#endif

#ifndef _RWSTD_NO_EXCEPTIONS
    catch(...)
    {
      bool flag = false;
      try {
        os.setstate(ios_base::badbit);
      }
      catch( ios_base::failure ) { flag= true; }
      if ( flag ) throw;
    }
#endif

    if ( err ) os.setstate(err);  

    return os;
  }

  /*
   * basic_ostream& operator<< (basic_ostream<_charT, _Traits>& os, const char *)
   *
   */

#ifndef _RWSTD_NO_OVERLOAD_OF_TEMPLATE_FUNCTION

  template<class _charT, class _Traits>
  basic_ostream<_charT, _Traits>&
  _RWSTDExportTemplate operator<< ( basic_ostream<_charT, _Traits>& os, const char *s)
  {
    ios_base::iostate err = 0;

#ifndef _RWSTD_NO_EXCEPTIONS
    try {
#endif
  
      if ( s ) {   
        _TYPENAME basic_ostream<_charT, _Traits>::sentry opfx(os);
        if (opfx)
        {
          int   dlen = (int)char_traits<char>::length(s);
          int   pad = (int)os.width() - dlen;

#ifndef _RWSTD_NO_TEMPLATE_ON_RETURN_TYPE
          const ctype<_charT>& ct = use_facet< ctype<_charT> >(os.getloc());
#else
          const ctype<_charT>& ct = use_facet(os.getloc(),(ctype<_charT>*)0);
#endif //  _RWSTD_NO_TEMPLATE_ON_RETURN_TYPE

          // place right padding
          if( !(os.flags() & ios_base::adjustfield) ||  (os.flags() & ios_base::right) )
          {
            while(--pad >= 0)
            {
              if( _Traits::eq_int_type(os.rdbuf()->sputc(os.fill()),_Traits::eof()))
              {
                err = ios_base::badbit;
                break;
              }
            }
          }
         
          // output internal padding
          if(os.good() && (os.flags() & ios_base::internal))
          {
            while(--pad >= 0) {
              if( _Traits::eq_int_type(os.rdbuf()->sputc(os.fill()),_Traits::eof()))
              {
                err = ios_base::badbit;
                break;
              }
            }
          }

          if(os.good() && dlen)
          {
            while ( dlen )
            {
              if( _Traits::eq_int_type(
                  os.rdbuf()->sputc(ct.widen(*s)),
                  _Traits::eof())) 
              { 
                err = ios_base::badbit;    
              }
              s++;
              dlen --;
            }
          }

          // output left padding. 
          if(os.good() && (os.flags() & ios_base::left))
          {
            while(--pad >= 0) {
              if( _Traits::eq_int_type(os.rdbuf()->sputc(os.fill()),_Traits::eof()))
              {
                err = ios_base::badbit;
                break;
              }
            }
          }

          os.width(0);
        }
      }
      else
        err = ios_base::badbit;

#ifndef _RWSTD_NO_EXCEPTIONS
    }
#endif

#ifndef _RWSTD_NO_EXCEPTIONS
    catch(...)
    {
      bool flag = false;
      try {
        os.setstate(ios_base::badbit);
      }
      catch( ios_base::failure ) { flag= true; }
      if ( flag ) throw;
    }
#endif // _RWSTD_NO_EXCEPTIONS

    if ( err ) os.setstate(err);  

    return os;
  }
#endif // _RWSTD_NO_OVERLOAD_OF_TEMPLATE_FUNCTION

  /*
   * basic_ostream& operator<< (basic_ostream<char, _Traits>& os, const char *)
   *
   */

#ifndef _RWSTD_NO_OVERLOAD_OF_TEMPLATE_FUNCTION

#ifndef _RWSTD_NO_FUNC_PARTIAL_SPEC
  template<class _Traits>
  basic_ostream<char, _Traits>&
  _RWSTDExportTemplate operator<< ( basic_ostream<char, _Traits>& os, const char *s)
  {
    ios_base::iostate err = 0;

#ifndef _RWSTD_NO_EXCEPTIONS
    try {
#endif
      if ( s )
      {   
        _TYPENAME basic_ostream<char, _Traits>::sentry opfx(os);
        if (opfx)
        {
          int   dlen = _Traits::length(s);
          int   pad = os.width() - dlen;

          // place right padding
          if( !(os.flags() & ios_base::adjustfield) ||  (os.flags() & ios_base::right) )
          {
            while(--pad >= 0)
            {
              if( _Traits::eq_int_type(os.rdbuf()->sputc(os.fill()),_Traits::eof()))
              {
                err = ios_base::badbit;
                break;
              }
            }
          }
         
          // output internal padding
          if(os.good() && (os.flags() & ios_base::internal))
          {
            while(--pad >= 0) {
              if( _Traits::eq_int_type(os.rdbuf()->sputc(os.fill()),_Traits::eof()))
              {
                err = ios_base::badbit;
                break;
              }
            }
          }

          if(os.good() && dlen) {
            if(os.rdbuf() && (os.rdbuf()->sputn(s, dlen) != dlen))
              err = ios_base::badbit;
          }

          // output left padding. 
          if(os.good() && (os.flags() & ios_base::left))
          {
            while(--pad >= 0) {
              if( _Traits::eq_int_type(os.rdbuf()->sputc(os.fill()),_Traits::eof()))
              {
                err = ios_base::badbit;
                break;
              }
            }
          }

          os.width(0);
        }
      }
      else
        err = ios_base::badbit;

#ifndef _RWSTD_NO_EXCEPTIONS
    }
#endif

#ifndef _RWSTD_NO_EXCEPTIONS
    catch(...)
    {
      bool flag = false;
      try {
        os.setstate(ios_base::badbit);
      }
      catch( ios_base::failure ) { flag= true; }
      if ( flag ) throw;
    }
#endif

    if ( err ) os.setstate(err);  

    return os;
  }
#endif // _RWSTD_NO_FUNC_PARTIAL_SPEC
#endif // _RWSTD_NO_OVERLOAD_OF_TEMPLATE_FUNCTION

  /*
   * basic_ostream& operator<<( basic_ostream<_charT, _Traits>&, _charT )
   */

  template<class _charT, class _Traits>
  basic_ostream<_charT, _Traits>&
  _RWSTDExportTemplate operator<< (basic_ostream<_charT, _Traits>& os, _charT c)
  {
    ios_base::iostate err = 0;

#ifndef _RWSTD_NO_EXCEPTIONS
    try {
#endif

      _TYPENAME basic_ostream<_charT, _Traits>::sentry opfx(os); 

      if(opfx) 
      {
        int   pad = (int)os.width() - 1;

        // place right padding
        if( !(os.flags() & ios_base::adjustfield) ||  (os.flags() & ios_base::right) )
        {
          while(--pad >= 0)
          {
            if( _Traits::eq_int_type(os.rdbuf()->sputc(os.fill()),_Traits::eof()))
            {
              err = ios_base::badbit;
              break;
            }
          }
        }
         
        // output internal padding
        if(os.good() && (os.flags() & ios_base::internal))
        {
          while(--pad >= 0)
          {
            if( _Traits::eq_int_type(os.rdbuf()->sputc(os.fill()),_Traits::eof()))
            {
              err = ios_base::badbit;
              break;
            }
          }
        }

        if(os.good())
        {
          if( _Traits::eq_int_type(os.rdbuf()->sputc(c),_Traits::eof())) 
            err = ios_base::badbit;    
        }
 
        // output left padding. 
        if(os.good() && (os.flags() & ios_base::left))
        {
          while(--pad >= 0)
          {
            if( _Traits::eq_int_type(os.rdbuf()->sputc(os.fill()),_Traits::eof()))
            {
              err = ios_base::badbit;
              break;
            }
          }
        }

        os.width(0);
     
      }

#ifndef _RWSTD_NO_EXCEPTIONS
    }
#endif

#ifndef _RWSTD_NO_EXCEPTIONS
    catch(...)
    {
      bool flag = false;
      try {
        os.setstate(ios_base::failbit);
      }
      catch( ios_base::failure ) { flag= true; }
      if ( flag ) throw;
    }
#endif
    if ( err ) os.setstate(err); 
    return os;
  }

  /*
   * basic_ostream& operator<<( basic_ostream<_charT, _Traits>&, char )
   */

#ifndef _RWSTD_NO_OVERLOAD_OF_TEMPLATE_FUNCTION

  template<class _charT, class _Traits>
  basic_ostream<_charT, _Traits>&
  _RWSTDExportTemplate operator<< (basic_ostream<_charT, _Traits>& os, char c)
  {
    ios_base::iostate err = 0;

#ifndef _RWSTD_NO_EXCEPTIONS
    try {
#endif
      _TYPENAME basic_ostream<_charT, _Traits>::sentry opfx(os); 

      if(opfx) 
      {
#ifndef _RWSTD_NO_TEMPLATE_ON_RETURN_TYPE
        const ctype<_charT>& ct = use_facet< ctype<_charT> >(os.getloc());
#else
        const ctype<_charT>& ct = use_facet(os.getloc(),(ctype<_charT>*)0);
#endif // _RWSTD_NO_TEMPLATE_ON_RETURN_TYPE

        int   pad = (int)os.width() - 1;

        // place right padding
        if( !(os.flags() & ios_base::adjustfield) ||  (os.flags() & ios_base::right) )
        {
          while(--pad >= 0)
          {
            if( _Traits::eq_int_type(os.rdbuf()->sputc(os.fill()),_Traits::eof()))
            {
              err = ios_base::badbit;
              break;
            }
          }
        }
         
        // output internal padding
        if(os.good() && (os.flags() & ios_base::internal))
        {
          while(--pad >= 0)
          {
            if( _Traits::eq_int_type(os.rdbuf()->sputc(os.fill()),_Traits::eof()))
            {
              err = ios_base::badbit;
              break;
            }
          }
        }

        if(os.good())
        {
          if( _Traits::eq_int_type(os.rdbuf()->sputc(ct.widen(c)),_Traits::eof())) 
            err = ios_base::badbit;      
        }
 
        // output left padding. 
        if(os.good() && (os.flags() & ios_base::left))
        {
          while(--pad >= 0)
          {
            if( _Traits::eq_int_type(os.rdbuf()->sputc(os.fill()),_Traits::eof()))
            {
              err = ios_base::badbit;
              break;
            }
          }
        }
        os.width(0);    
      }

#ifndef _RWSTD_NO_EXCEPTIONS
    }
#endif

#ifndef _RWSTD_NO_EXCEPTIONS
    catch(...)
    {
      bool flag = false;
      try {
        os.setstate(ios_base::failbit);
      }
      catch( ios_base::failure ) { flag= true; }
      if ( flag ) throw;
    }
#endif

    if ( err ) os.setstate(err); 

    return os;
  }
#endif // _RWSTD_NO_OVERLOAD_OF_TEMPLATE_FUNCTION 

  /*
   * basic_ostream& operator<<( basic_ostream<char, _Traits>&, char )
   *
   */

#ifndef _RWSTD_NO_OVERLOAD_OF_TEMPLATE_FUNCTION
#ifndef _RWSTD_NO_FUNC_PARTIAL_SPEC

  template<class _Traits>
  basic_ostream<char, _Traits>&
  _RWSTDExportTemplate operator<< (basic_ostream<char, _Traits>& os, char c)
  {
    ios_base::iostate err = 0;

#ifndef _RWSTD_NO_EXCEPTIONS
    try {
#endif

      _TYPENAME basic_ostream<char, _Traits>::sentry opfx(os); 

      if(opfx) 
      {
        int   pad = os.width() - 1;

        // place right padding
        if( !(os.flags() & ios_base::adjustfield) ||  (os.flags() & ios_base::right) )
        {
          while(--pad >= 0)
          {
            if( _Traits::eq_int_type(os.rdbuf()->sputc(os.fill()),_Traits::eof()))
            {
              err = ios_base::badbit;
              break;
            }
          }
        }
         
        // output internal padding
        if(os.good() && (os.flags() & ios_base::internal))
        {
          while(--pad >= 0)
          {
            if( _Traits::eq_int_type(os.rdbuf()->sputc(os.fill()),_Traits::eof()))
            {
              err = ios_base::badbit;
              break;
            }
          }
        }

        if(os.good()) {
          if( _Traits::eq_int_type(os.rdbuf()->sputc(c),_Traits::eof())) 
            err = ios_base::badbit;       
        }
 
        // output left padding. 
        if(os.good() && (os.flags() & ios_base::left))
        {
          while(--pad >= 0)
          {
            if( _Traits::eq_int_type(os.rdbuf()->sputc(os.fill()),_Traits::eof()))
            {
              err = ios_base::badbit;
              break;
            }
          }
        }
        os.width(0);    
      }

#ifndef _RWSTD_NO_EXCEPTIONS
    }
#endif

#ifndef _RWSTD_NO_EXCEPTIONS
    catch(...)
    {
      bool flag = false;
      try {
        os.setstate(ios_base::failbit);
      }
      catch( ios_base::failure ) { flag= true; }
      if ( flag ) throw;
    }
#endif

    if ( err ) os.setstate(err); 

    return os;
  }
#endif // _RWSTD_NO_FUNC_PARTIAL_SPEC 
#endif // _RWSTD_NO_OVERLOAD_OF_TEMPLATE_FUNCTION 
 
#ifndef _RWSTD_NO_SIGNED_CHAR_IN_STREAMS

  /*
   * ostream& operator<<(basic_ostream<char,_Traits>&,unsigned char )
   */

  template <class _Traits>
  basic_ostream<char, _Traits>&
  _RWSTDExportTemplate operator<<(basic_ostream<char, _Traits>& os,unsigned char uc)
  {
    return (os << (char)uc);
  }

  /*
   * ostream& operator<<(basic_ostream<char, _Traits>&,signed char )
   */

  template <class _Traits>
  basic_ostream<char, _Traits>&
  _RWSTDExportTemplate operator<<(basic_ostream<char, _Traits>& os,signed char sc)
  {
    return (os << (char)sc);
  }

  /*
   * ostream& operator<<(basic_ostream<char, _Traits>&,const unsigned char* )
   */

  template <class _Traits>
  basic_ostream<char, _Traits>&
  _RWSTDExportTemplate operator<<(basic_ostream<char, _Traits>& os,const unsigned char* uc)
  {
    return (os << (char *)uc);
  }

  /*
   * ostream& operator<<(basic_ostream<char, _Traits>&,const signed char* )
   */

  template <class _Traits>
  basic_ostream<char, _Traits>&
  _RWSTDExportTemplate operator<<(basic_ostream<char, _Traits>& os,const signed char* sc)
  {
    return (os << (char *)sc);
  }
#endif // _RWSTD_NO_SIGNED_CHAR_IN_STREAMS


  /*
   * basic_ostream& operator<<(bool n)
   */

#ifndef _RWSTD_NO_BOOL

  template<class _charT, class _Traits>
  basic_ostream<_charT, _Traits>& 
  basic_ostream<_charT, _Traits>::operator<<(bool n)
  {
    ios_base::iostate err = 0;

#ifndef _RWSTD_NO_EXCEPTIONS
    try {
#endif

      _TYPENAME basic_ostream<_charT, _Traits>::sentry opfx(*this);
      if (opfx)
      {
        if (
#ifndef _RWSTD_NO_TEMPLATE_ON_RETURN_TYPE
            use_facet<num_put<_charT,ostreambuf_iterator<_charT,_Traits> > >(this->getloc())
#else
            use_facet(this->getloc(),(num_put<_charT,ostreambuf_iterator<_charT,_Traits> >*)0)
#endif // _RWSTD_NO_TEMPLATE_ON_RETURN_TYPE
            .put(ostreambuf_iterator<_charT,_Traits>(*this),*this,this->fill(),n).failed() )
          err = ios_base::badbit;
        this->width(0);
      }

#ifndef _RWSTD_NO_EXCEPTIONS
    }
#endif

#ifndef _RWSTD_NO_EXCEPTIONS
    catch(...)
    {
      bool flag = false;
      try {
        this->setstate(ios_base::badbit);
      }
      catch( ios_base::failure ) { flag= true; }
      if ( flag ) throw;
    }
#endif // _RWSTD_NO_EXCEPTIONS 
    if ( err ) this->setstate(err);
    return *this;
  }

#endif // _RWSTD_NO_BOOL

  /*
   * basic_ostream& operator<<(short)
   */

  template<class _charT, class _Traits>
  basic_ostream<_charT, _Traits>&
  basic_ostream<_charT, _Traits>::operator<<(short n)
  {
    ios_base::iostate err = 0;

#ifndef _RWSTD_NO_EXCEPTIONS
    try {
#endif  
      _TYPENAME basic_ostream<_charT, _Traits>::sentry opfx(*this);
      if (opfx)
      {
        if (
#ifndef _RWSTD_NO_TEMPLATE_ON_RETURN_TYPE
            use_facet<num_put<_charT,ostreambuf_iterator<_charT,_Traits> > >(this->getloc())
#else
            use_facet(this->getloc(),(num_put<_charT,ostreambuf_iterator<_charT,_Traits> >*)0)
#endif
#ifndef _RWSTD_STRICT_ANSI
            .put(ostreambuf_iterator<_charT,_Traits>(*this),*this,basic_ios<_charT,_Traits>::fill(),n).failed() )
#else
            .put(ostreambuf_iterator<_charT,_Traits>(*this),*this,basic_ios<_charT,_Traits>::fill(),(long)n).failed() )
#endif
          err = ios_base::badbit;
        this->width(0);
      }
 
#ifndef _RWSTD_NO_EXCEPTIONS
    }
#endif

#ifndef _RWSTD_NO_EXCEPTIONS
    catch(...)
    {
      bool flag = false;
      try {
        this->setstate(ios_base::badbit);
      }
      catch( ios_base::failure ) { flag= true; }
      if ( flag ) throw;
    }
#endif
    if ( err ) this->setstate(err);
    return *this;
  }

  /*
   * basic_ostream& operator<<(unsigned short)
   */

  template<class _charT, class _Traits>
  basic_ostream<_charT, _Traits>&
  basic_ostream<_charT, _Traits>::operator<<(unsigned short n)
  {
    ios_base::iostate err = 0;

#ifndef _RWSTD_NO_EXCEPTIONS
    try {
#endif  
      _TYPENAME basic_ostream<_charT, _Traits>::sentry opfx(*this);
      if (opfx)
      {
        if (
#ifndef _RWSTD_NO_TEMPLATE_ON_RETURN_TYPE
            use_facet<num_put<_charT,ostreambuf_iterator<_charT,_Traits> > >(this->getloc())
#else
            use_facet(this->getloc(),(num_put<_charT,ostreambuf_iterator<_charT,_Traits> >*)0)
#endif // _RWSTD_NO_TEMPLATE_ON_RETURN_TYPE
#ifndef _RWSTD_STRICT_ANSI
            .put(ostreambuf_iterator<_charT,_Traits>(*this),*this,basic_ios<_charT,_Traits>::fill(),n).failed() )
#else
            .put(ostreambuf_iterator<_charT,_Traits>(*this),*this,basic_ios<_charT,_Traits>::fill(),(unsigned long)n).failed() )
#endif
          err = ios_base::badbit;
        this->width(0);
      }

#ifndef _RWSTD_NO_EXCEPTIONS
    }
#endif

#ifndef _RWSTD_NO_EXCEPTIONS
    catch(...)
    {
      bool flag = false;
      try {
        this->setstate(ios_base::badbit);
      }
      catch( ios_base::failure ) { flag= true; }
      if ( flag ) throw;
    }
#endif // _RWSTD_NO_EXCEPTIONS 
    if ( err ) this->setstate(err);
    return *this; 
  }


  /*
   * basic_ostream& operator<<(int)
   */

  template<class _charT, class _Traits>
  basic_ostream<_charT, _Traits>&
  basic_ostream<_charT, _Traits>::operator<<(int n)
  {
    ios_base::iostate err = 0;

#ifndef _RWSTD_NO_EXCEPTIONS
    try {
#endif  

      _TYPENAME basic_ostream<_charT, _Traits>::sentry opfx(*this);
      if (opfx)
      {
        if ( 
#ifndef _RWSTD_NO_TEMPLATE_ON_RETURN_TYPE
            use_facet<num_put<_charT,ostreambuf_iterator<_charT,_Traits> > >(this->getloc())
#else
            use_facet(this->getloc(),(num_put<_charT,ostreambuf_iterator<_charT,_Traits> >*)0)
#endif // _RWSTD_NO_TEMPLATE_ON_RETURN_TYPE
#ifndef _RWSTD_STRICT_ANSI
            .put(ostreambuf_iterator<_charT,_Traits>(*this),*this,basic_ios<_charT,_Traits>::fill(),n).failed() )
#else
            .put(ostreambuf_iterator<_charT,_Traits>(*this),*this,basic_ios<_charT,_Traits>::fill(),(long)n).failed() )
#endif
          err = ios_base::badbit;
        this->width(0);
      }

#ifndef _RWSTD_NO_EXCEPTIONS
    }
#endif

#ifndef _RWSTD_NO_EXCEPTIONS
    catch(...)
    {
      bool flag = false;
      try {
        this->setstate(ios_base::badbit);
      }
      catch( ios_base::failure ) { flag= true; }
      if ( flag ) throw;
    }
#endif
    if ( err ) this->setstate(err);  
    return *this; 
  }
  /*
   * basic_ostream& operator<<(unsigned int)
   */

  template<class _charT, class _Traits>
  basic_ostream<_charT, _Traits>&
  basic_ostream<_charT, _Traits>::operator<<(unsigned int n)
  {
    ios_base::iostate err = 0;

#ifndef _RWSTD_NO_EXCEPTIONS
    try {
#endif 

      _TYPENAME basic_ostream<_charT, _Traits>::sentry opfx(*this);
      if (opfx)
      {
        if (
#ifndef _RWSTD_NO_TEMPLATE_ON_RETURN_TYPE
            use_facet<num_put<_charT,ostreambuf_iterator<_charT,_Traits> > >(this->getloc())
#else
            use_facet(this->getloc(),(num_put<_charT,ostreambuf_iterator<_charT,_Traits> >*)0)
#endif // _RWSTD_NO_TEMPLATE_ON_RETURN_TYPE
#ifndef _RWSTD_STRICT_ANSI
            .put(ostreambuf_iterator<_charT,_Traits>(*this),*this,basic_ios<_charT,_Traits>::fill(),n).failed() )
#else
            .put(ostreambuf_iterator<_charT,_Traits>(*this),*this,basic_ios<_charT,_Traits>::fill(),(unsigned long)n).failed() )
#endif
          err = ios_base::badbit;
        this->width(0);
      }

#ifndef _RWSTD_NO_EXCEPTIONS
    }
#endif

#ifndef _RWSTD_NO_EXCEPTIONS
    catch(...)
    {
      bool flag = false;
      try {
        this->setstate(ios_base::badbit);
      }
      catch( ios_base::failure ) { flag= true; }
      if ( flag ) throw;
    }
#endif
    if ( err ) this->setstate(err);
    return *this; 
  }

  /*
   * basic_ostream& operator<<(long)
   */

  template<class _charT, class _Traits>
  basic_ostream<_charT, _Traits>&
  basic_ostream<_charT, _Traits>::operator<<(long n)
  {
    ios_base::iostate err = 0;

#ifndef _RWSTD_NO_EXCEPTIONS
    try {
#endif

      _TYPENAME basic_ostream<_charT, _Traits>::sentry opfx(*this);
      if (opfx)
      {
        if (
#ifndef _RWSTD_NO_TEMPLATE_ON_RETURN_TYPE
            use_facet<num_put<_charT,ostreambuf_iterator<_charT,_Traits> > >(this->getloc())
#else
            use_facet(this->getloc(),(num_put<_charT,ostreambuf_iterator<_charT,_Traits> >*)0)
#endif
            .put(ostreambuf_iterator<_charT,_Traits>(*this),*this,basic_ios<_charT,_Traits>::fill(),n).failed() )
          err = ios_base::badbit;
        this->width(0);
      } 

#ifndef _RWSTD_NO_EXCEPTIONS
    }
#endif

#ifndef _RWSTD_NO_EXCEPTIONS
    catch(...)
    {
      bool flag = false;
      try {
        this->setstate(ios_base::badbit);
      }
      catch( ios_base::failure ) { flag= true; }
      if ( flag ) throw;
    }
#endif // _RWSTD_NO_EXCEPTIONS 
    if ( err ) this->setstate(err);
    return *this; 
  }

  /*
   * basic_ostream& operator<<(unsigned long)
   */

  template<class _charT, class _Traits>
  basic_ostream<_charT, _Traits>&
  basic_ostream<_charT, _Traits>::operator<<(unsigned long n)
  {
    ios_base::iostate err = 0;

#ifndef _RWSTD_NO_EXCEPTIONS
    try {
#endif
      _TYPENAME basic_ostream<_charT, _Traits>::sentry opfx(*this);
      if (opfx)
      {
        if (
#ifndef _RWSTD_NO_TEMPLATE_ON_RETURN_TYPE
            use_facet<num_put<_charT,ostreambuf_iterator<_charT,_Traits> > >(this->getloc())
#else
            use_facet(this->getloc(),(num_put<_charT,ostreambuf_iterator<_charT,_Traits> >*)0)
#endif
            .put(ostreambuf_iterator<_charT,_Traits>(*this),*this,basic_ios<_charT,_Traits>::fill(),n).failed() )
          err = ios_base::badbit;
        this->width(0);
      }

#ifndef _RWSTD_NO_EXCEPTIONS
    }
#endif

#ifndef _RWSTD_NO_EXCEPTIONS
    catch(...)
    {
      bool flag = false;
      try {
        this->setstate(ios_base::badbit);
      }
      catch( ios_base::failure ) { flag= true; }
      if ( flag ) throw;
    }
#endif
    if ( err ) this->setstate(err);
    return *this; 
  }

  /*
   * basic_ostream& operator<<(float)
   */

  template<class _charT, class _Traits>
  basic_ostream<_charT, _Traits>&
  basic_ostream<_charT, _Traits>::operator<<(float n)
  {
    ios_base::iostate err = 0;

#ifndef _RWSTD_NO_EXCEPTIONS
    try {
#endif

      _TYPENAME basic_ostream<_charT, _Traits>::sentry opfx(*this);
      if (opfx)
      {
        if (
#ifndef _RWSTD_NO_TEMPLATE_ON_RETURN_TYPE
            use_facet<num_put<_charT,ostreambuf_iterator<_charT,_Traits> > >(this->getloc())
#else
            use_facet(this->getloc(),(num_put<_charT,ostreambuf_iterator<_charT,_Traits> >*)0)
#endif // _RWSTD_NO_TEMPLATE_ON_RETURN_TYPE
            .put(ostreambuf_iterator<_charT,_Traits>(*this),*this,basic_ios<_charT,_Traits>::fill(),(double)n).failed() )
          err = ios_base::badbit;
        this->width(0);
      }

#ifndef _RWSTD_NO_EXCEPTIONS
    }
#endif

#ifndef _RWSTD_NO_EXCEPTIONS
    catch(...)
    {
      bool flag = false;
      try {
        this->setstate(ios_base::badbit);
      }
      catch( ios_base::failure ) { flag= true; }
      if ( flag ) throw;
    }
#endif
    if ( err ) this->setstate(err);
    return *this;
  }

  /*
   * basic_ostream& operator<<(double)
   */

  template<class _charT, class _Traits>
  basic_ostream<_charT, _Traits>&
  basic_ostream<_charT, _Traits>::operator<<(double n)
  {
    ios_base::iostate err = 0;

#ifndef _RWSTD_NO_EXCEPTIONS
    try {
#endif

      _TYPENAME basic_ostream<_charT, _Traits>::sentry opfx(*this);
      if (opfx)
      {
        if (
#ifndef _RWSTD_NO_TEMPLATE_ON_RETURN_TYPE
            use_facet<num_put<_charT,ostreambuf_iterator<_charT,_Traits> > >(this->getloc())
#else
            use_facet(this->getloc(),(num_put<_charT,ostreambuf_iterator<_charT,_Traits> >*)0)
#endif // _RWSTD_NO_TEMPLATE_ON_RETURN_TYPE
            .put(ostreambuf_iterator<_charT,_Traits>(*this),*this,basic_ios<_charT,_Traits>::fill(),n).failed() )
          err = ios_base::badbit;
        this->width(0);
      }

#ifndef _RWSTD_NO_EXCEPTIONS
    }
#endif

#ifndef _RWSTD_NO_EXCEPTIONS
    catch(...)
    {
      bool flag = false;
      try {
        this->setstate(ios_base::badbit);
      }
      catch( ios_base::failure ) { flag= true; }
      if ( flag ) throw;
    }
#endif
    if ( err ) this->setstate(err);
    return *this;
  }

  /*
   * basic_ostream& operator<<(long double)
   */

  template<class _charT, class _Traits>
  basic_ostream<_charT, _Traits>&
  basic_ostream<_charT, _Traits>::operator<<(long double n)
  {
    ios_base::iostate err = 0;

#ifndef _RWSTD_NO_EXCEPTIONS
    try {
#endif
      _TYPENAME basic_ostream<_charT, _Traits>::sentry opfx(*this);
      if (opfx)
      {
        if (
#ifndef _RWSTD_NO_TEMPLATE_ON_RETURN_TYPE
            use_facet<num_put<_charT,ostreambuf_iterator<_charT,_Traits> > >(this->getloc())
#else
            use_facet(this->getloc(),(num_put<_charT,ostreambuf_iterator<_charT,_Traits> >*)0)
#endif // _RWSTD_NO_TEMPLATE_ON_RETURN_TYPE
            .put(ostreambuf_iterator<_charT,_Traits>(*this),*this,basic_ios<_charT,_Traits>::fill(),n).failed() )
          err = ios_base::badbit;
        this->width(0);
      }

#ifndef _RWSTD_NO_EXCEPTIONS
    }
#endif

#ifndef _RWSTD_NO_EXCEPTIONS
    catch(...)
    {
      bool flag = false;
      try {
        this->setstate(ios_base::badbit);
      }
      catch( ios_base::failure ) { flag= true; }
      if ( flag ) throw;
    }
#endif
    if ( err ) this->setstate(err);
    return *this;
  }

/*
 * basic_ostream& operator<<(_RWSTD_LONG_LONG)
 */

#ifdef _RWSTD_LONG_LONG

  template<class _charT, class _Traits>
  basic_ostream<_charT, _Traits>&
  basic_ostream<_charT, _Traits>::operator<<(_RWSTD_LONG_LONG n)
  {
    ios_base::iostate err = 0;

#ifndef _RWSTD_NO_EXCEPTIONS
    try {
#endif

      _TYPENAME basic_ostream<_charT, _Traits>::sentry opfx(*this);
      if (opfx)
      {
	if (
#ifndef _RWSTD_NO_TEMPLATE_ON_RETURN_TYPE
	    use_facet<num_put<_charT,ostreambuf_iterator<_charT,_Traits> > >(this->getloc())
#else
	    use_facet(this->getloc(),(num_put<_charT,ostreambuf_iterator<_charT,_Traits> >*)0)
#endif
	    .put(ostreambuf_iterator<_charT,_Traits>(*this),*this,basic_ios<_charT,_Traits>::fill(),n).failed() )
	  err = ios_base::badbit;
   
	this->width(0);

      }

#ifndef _RWSTD_NO_EXCEPTIONS
    }
#endif

#ifndef _RWSTD_NO_EXCEPTIONS
    catch(...)
    {
      bool flag = false;
      try {
	this->setstate(ios_base::badbit);
      }
      catch( ios_base::failure ) { flag= true; }
      if ( flag ) throw;
    }
#endif

    if ( err ) this->setstate(err);

    return *this;
  }

/*
 * basic_ostream& operator<<(unsigned _RWSTD_LONG_LONG)
 */

  template<class _charT, class _Traits>
  basic_ostream<_charT, _Traits>&
  basic_ostream<_charT, _Traits>::operator<<(unsigned _RWSTD_LONG_LONG n)
  {
    ios_base::iostate err = 0;

#ifndef _RWSTD_NO_EXCEPTIONS
    try {
#endif

      _TYPENAME basic_ostream<_charT, _Traits>::sentry opfx(*this);
      if (opfx)
      {
	if (
#ifndef _RWSTD_NO_TEMPLATE_ON_RETURN_TYPE
	    use_facet<num_put<_charT,ostreambuf_iterator<_charT,_Traits> > >(this->getloc())
#else
	    use_facet(this->getloc(),(num_put<_charT,ostreambuf_iterator<_charT,_Traits> >*)0)
#endif
	    .put(ostreambuf_iterator<_charT,_Traits>(*this),*this,basic_ios<_charT,_Traits>::fill(),n).failed() )
	  err = ios_base::badbit;
   
	this->width(0);

      }

#ifndef _RWSTD_NO_EXCEPTIONS
    }
#endif

#ifndef _RWSTD_NO_EXCEPTIONS
    catch(...)
    {
      bool flag = false;
      try {
	this->setstate(ios_base::badbit);
      }
      catch( ios_base::failure ) { flag= true; }
      if ( flag ) throw;
    }
#endif

    if ( err ) this->setstate(err);

    return *this;
  }
#endif //_RWSTD_LONG_LONG



  /*
   * basic_ostream& operator<<(const void *)
   */

  template<class _charT, class _Traits>
  basic_ostream<_charT, _Traits>&
  basic_ostream<_charT, _Traits>::operator<<(const void *p)
  {
    ios_base::iostate err = 0;

#ifndef _RWSTD_NO_EXCEPTIONS
    try {
#endif

      _TYPENAME basic_ostream<_charT, _Traits>::sentry opfx(*this);
      if (opfx)
      {
        if (
#ifndef _RWSTD_NO_TEMPLATE_ON_RETURN_TYPE
            use_facet<num_put<_charT,ostreambuf_iterator<_charT,_Traits> > >(this->getloc())
#else
            use_facet(this->getloc(),(num_put<_charT,ostreambuf_iterator<_charT,_Traits> >*)0)
#endif // _RWSTD_NO_TEMPLATE_ON_RETURN_TYPE
            .put(ostreambuf_iterator<_charT,_Traits>(*this),*this,this->fill(),p).failed() )
          err = ios_base::badbit;
        this->width(0);
      }

#ifndef _RWSTD_NO_EXCEPTIONS
    }
#endif

#ifndef _RWSTD_NO_EXCEPTIONS
    catch(...)
    {
      bool flag = false;
      try {
        this->setstate(ios_base::badbit);
      }
      catch( ios_base::failure ) { flag= true; }
      if ( flag ) throw;
    }
#endif
    if ( err ) this->setstate(err);
    return *this;
  }

  /*
   * basic_ostream& put(char_type)
   */

  template<class _charT, class _Traits>
  basic_ostream<_charT, _Traits>&
  basic_ostream<_charT, _Traits>::put(char_type c)
  {
    ios_base::iostate err = 0; 

#ifndef _RWSTD_NO_EXCEPTIONS
    try {
#endif
      _TYPENAME basic_ostream<_charT, _Traits>::sentry opfx(*this);
      if(opfx)
      {
        if( _Traits::eq_int_type(this->rdbuf()->sputc(c),_Traits::eof()) )  
          err = ios_base::badbit;    
      }

#ifndef _RWSTD_NO_EXCEPTIONS
    }
#endif

#ifndef _RWSTD_NO_EXCEPTIONS
    catch(...)
    {
      bool flag = false;
      try {
        this->setstate(ios_base::badbit);
      }
      catch( ios_base::failure ) { flag= true; }
      if ( flag ) throw;
    }
#endif
    if ( err ) this->setstate(err);
    return *this;
  }

  /*
   * basic_ostream& write(const char_type *, streamsize)
   */

  template<class _charT, class _Traits>
  basic_ostream<_charT, _Traits>&
  basic_ostream<_charT, _Traits>::write(const char_type *s, streamsize n)
  {
    ios_base::iostate err = 0; 

#ifndef _RWSTD_NO_EXCEPTIONS
    try {
#endif 

      if(s)
      {
        _TYPENAME basic_ostream<_charT, _Traits>::sentry opfx(*this);
        if(opfx)
        {
          if(this->rdbuf()->sputn(s, n)!=n) 
            err = ios_base::badbit;
        }
      }
      else
        err = ios_base::badbit;

#ifndef _RWSTD_NO_EXCEPTIONS
    }
#endif

#ifndef _RWSTD_NO_EXCEPTIONS
    catch(...)
    {
      bool flag = false;
      try {
        this->setstate(ios_base::badbit);
      }
      catch( ios_base::failure ) { flag= true; }
      if ( flag ) throw;
    }
#endif
    if ( err ) this->setstate(err);
    return *this;
  }


  template<class _charT, class _Traits>
  basic_ostream<_charT, _Traits>&
  basic_ostream<_charT, _Traits>::seekp(off_type off, ios_base::seekdir dir)
  {
#ifdef _RWSTD_MULTI_THREAD
    if ( this->rdbuf() ) {
      _RWSTDGuard guard(this->rdbuf()->buffer_mutex_);
#endif
    if ( this->fail() ) return *this;
    this->rdbuf()->pubseekoff(off, dir, ios_base::out);

#ifdef _RWSTD_MULTI_THREAD
    }
#endif
    return *this;
  }

  template<class _charT, class _Traits>
  _TYPENAME basic_ostream<_charT,_Traits>::pos_type
  basic_ostream<_charT, _Traits>::tellp()
  {
    pos_type    p;

#ifdef _RWSTD_MULTI_THREAD
    if ( this->rdbuf() ) {
      _RWSTDGuard guard(this->rdbuf()->buffer_mutex_);
#endif // _RWSTD_MULTI_THREAD 
 
    if ( this->fail() ) return pos_type(-1);

    p = this->rdbuf()->pubseekoff(0, ios_base::cur,ios_base::out);

#ifdef _RWSTD_MULTI_THREAD
    }
#endif // _RWSTD_MULTI_THREAD 
    return p;
  }

#ifdef __linux__
  /*
   * String insertor
   */
  template<class _charT, class _Traits, class _Allocator>
  basic_ostream<_charT, _Traits>&
  operator<< (basic_ostream<_charT,_Traits>& os,
	      const basic_string<_charT,_Traits,_Allocator>& s)
  {
    ios_base::iostate err = 0;

#ifndef _RWSTD_NO_EXCEPTIONS
    try {
#endif
  
      _TYPENAME basic_ostream<_charT, _Traits>::sentry opfx(os);
      if (opfx)
      {
	int   dlen = (int)s.length();
	int   pad =  (int)os.width() - dlen;

	// place right padding
	if( !(os.flags() & ios_base::adjustfield) ||  os.flags() & ios_base::right )
	{
	  while(--pad >= 0)
	  {
	    if( _Traits::eq_int_type(os.rdbuf()->sputc(os.fill()),_Traits::eof())) {
	      err = ios_base::badbit;
	      break;
	    }
	  }
	}
         
	// output internal padding
	if(os.good() && (os.flags() & ios_base::internal))
	{
	  while(--pad >= 0)
	  {
	    if( _Traits::eq_int_type(os.rdbuf()->sputc(os.fill()),_Traits::eof()))
	    {
	      err = ios_base::badbit;
	      break;
	    }
	  }
	}

	if(os.good() && dlen) {
	  if(os.rdbuf() && (os.rdbuf()->sputn(s.c_str(), dlen) != dlen))
	    err = ios_base::badbit;
	}

	// output left padding. 
	if(os.good() && (os.flags() & ios_base::left))
	{
	  while(--pad >= 0)
	  {
	    if( _Traits::eq_int_type(os.rdbuf()->sputc(os.fill()),_Traits::eof()))
	    {
	      err = ios_base::badbit;
	      break;
	    }
	  }
	}

	os.width(0);
      }

#ifndef _RWSTD_NO_EXCEPTIONS
    }
#endif

#ifndef _RWSTD_NO_EXCEPTIONS
    catch(...)
    {
      bool flag = false;
      try {
        os.setstate(ios_base::badbit);
      }
      catch( ios_base::failure ) { flag= true; }
      if ( flag ) throw;
    }
#endif

    if ( err ) os.setstate(err);  

    return os;
  }
#endif /* __linux__ */

#ifndef _RWSTD_NO_NAMESPACE
}
#endif






