/*
   Copyright 10/07/02 Sun Microsystems, Inc.  All Rights Reserved.
*/
 
#ifndef __SUNINTERVAL_H
#define __SUNINTERVAL_H

#pragma ident "@(#)suninterval.h 1.3 02/10/07 Sun Microsystems, Inc."

#include <math.h>
#include <iostream.h>
#include <sunintervalimpl.h> // implementation private

/*****************************************************************************/

#if __cplusplus >= 199711
namespace SUNW_interval {
typedef bool interval_bool;
#else
typedef int interval_bool;
#endif

template< typename T > struct interval;

template<> struct interval<float>;
template<> struct interval<double>;
template<> struct interval<long double>;

/*****************************************************************************/

template<>
struct interval<float> : __r_interval
{
    /* Constructors */

    interval() { }

    interval( __r_interval v ) : __r_interval( v ) { }

    //       interval( const interval<float>& ) is default
    explicit interval( const interval<double>& v ) {
        __ri_interval_di_( (__r_interval*)this, (__d_interval*)&v );
    }
    explicit interval( const interval<long double>& v ) {
        __ri_interval_qi_( (__r_interval*)this, (__q_interval*)&v );
    }

    explicit interval( int v ) {
        __ri_interval_i_( (__r_interval*)this, v );
    }
    explicit interval( long long v ) {
        __ri_interval_xl_( (__r_interval*)this, v );
    }
    explicit interval( float v ) {
        __ri_interval_r_( (__r_interval*)this, v );
    }
    explicit interval( double v ) {
        __ri_interval_d_( (__r_interval*)this, v );
    }
    explicit interval( long double v ) {
        __ri_interval_q_( (__r_interval*)this, v );
    }

    interval( int v1, int v2 ) {
        __ri_interval_i_i_( (__r_interval*)this, v1, v2 );
    }
    interval( long long v1, long long v2 ) {
        __ri_interval_xl_xl_( (__r_interval*)this, v1, v2 );
    }
    interval( float v1, float v2 ) {
        __ri_interval_r_r_( (__r_interval*)this, v1, v2 );
    }
    interval( double v1, double v2 ) {
        __ri_interval_d_d_( (__r_interval*)this, v1, v2 );
    }
    interval( long double v1, long double v2 ) {
        __ri_interval_q_q_( (__r_interval*)this, v1, v2 );
    }

    explicit interval( const char* value ); 

    interval<float>& operator += ( interval<float> v) {
        *this = __ri_add( *this, v );
        return *this;
    }
    interval<float>& operator -= ( interval<float> v ) {
        *this = __ri_sub( *this, v );
        return *this;
    }
    interval<float>& operator *= ( interval<float> v ) {
        *this = __ri_mul( *this, v );
        return *this;
    }
    interval<float>& operator /= ( interval<float> v ) {
        *this = __ri_div( *this, v );
        return *this;
    }

    /* Extraction Functions: infimim, supremum */

    float inf() const { return l; }
    float sup() const { return u; }
};

/* Extraction Functions: infimim, supremum, midpoint, width, magnitude,
mignitude */

inline float inf( interval<float> v ) { return v.inf(); }
inline float sup( interval<float> v ) { return v.sup(); }
inline float mid( interval<float> v ) { return __ri_mid( v ); }
inline float wid( interval<float> v ) { return __ri_wid( v ); }
inline float mag( interval<float> v ) { return __ri_mag( v ); }
inline float mig( interval<float> v ) { return __ri_mig( v ); }

/* Arithmetic Operations: addition, subtraction, multiplication,
division, negation */

inline interval<float> operator + ( interval<float> v1, interval<float> v2 ) {
    return __ri_add( v1, v2 );
}
inline interval<float> operator - ( interval<float> v1, interval<float> v2 ) {
    return __ri_sub( v1, v2 );
}
inline interval<float> operator * ( interval<float> v1, interval<float> v2 ) {
    return __ri_mul( v1, v2 );
}
inline interval<float> operator / ( interval<float> v1, interval<float> v2 ) {
    return __ri_div( v1, v2 );
}
inline interval<float> operator - ( interval<float> v ) {
    return __ri_neg( v );
}

/* Intersection Division */

inline interval<float> 
divix( interval<float> v1, interval<float> v2, interval<float> v3 ) {
    return __ri_divx( v1, v2, v3 );
}

/* Hull Functions: regular hull, intersection */

inline interval<float> interval_hull( interval<float> v1, interval<float> v2 ) {
    return __ri_ih( v1, v2 );
}
inline interval<float> intersect( interval<float> v1, interval<float> v2 ) {
    return __ri_ix( v1, v2 );
}

/* Query Functions: is empty, ndigits */

inline interval_bool is_empty( interval<float> v ) {
    return __ri_isempty( v );
}
inline int ndigits( interval<float> v ) {
    return __ri_ndigits( v );
}

/* Membership Functions */

inline interval_bool in( int c, interval<float> v ) {
    return __i_in_ri( c, v );
}
inline interval_bool in( long long c, interval<float> v ) {
    return __xl_in_ri( c, v );
}
inline interval_bool in( float c, interval<float> v ) {
    return __r_in_ri( c, v );
}
inline interval_bool in( double c, interval<float> v ) {
    return __d_in_ri( c, v );
}
inline interval_bool in( long double c, interval<float> v ) {
    return __q_in_ri( c, v );
}

/* Comparison Functions: disjoint, in interior, subset, superset,
proper subset, proper superset */

inline interval_bool disjoint( interval<float> v1, interval<float> v2 ) {
    return __ri_dj( v1, v2 );
}
inline interval_bool in_interior( interval<float> v1, interval<float> v2 ) {
    return __ri_int( v1, v2 );
}
inline interval_bool subset( interval<float> v1, interval<float> v2 ) {
    return __ri_sb( v1, v2 );
}
inline interval_bool superset( interval<float> v1, interval<float> v2 ) {
    return __ri_sp( v1, v2 );
}
inline interval_bool proper_subset( interval<float> v1, interval<float> v2 ) {
    return __ri_psb( v1, v2 );
}
inline interval_bool proper_superset( interval<float> v1, interval<float> v2 ) {
    return __ri_psp( v1, v2 );
}

/* Set Relations: equal, not equal, less than, less than or equal,
greater than, greater than or equal */

inline interval_bool seq( interval<float> v1, interval<float> v2 ) {
    return __ri_seq( v1, v2 );
}
inline interval_bool operator ==( interval<float> v1, interval<float> v2 ) {
    return __ri_seq( v1, v2 );
}
inline interval_bool sne( interval<float> v1, interval<float> v2 ) {
    return __ri_sne( v1, v2 );
}
inline interval_bool operator !=( interval<float> v1, interval<float> v2 ) {
    return __ri_sne( v1, v2 );
}
inline interval_bool slt( interval<float> v1, interval<float> v2 ) {
    return __ri_slt( v1, v2 );
}
inline interval_bool sle( interval<float> v1, interval<float> v2 ) {
    return __ri_sle( v1, v2 );
}
inline interval_bool sgt( interval<float> v1, interval<float> v2 ) {
    return __ri_sgt( v1, v2 );
}
inline interval_bool sge( interval<float> v1, interval<float> v2 ) {
    return __ri_sge( v1, v2 );
}

/* Certainly Relations: equal, not equal, less than, less than or
equal, greater than, greater than or equal */

inline interval_bool ceq( interval<float> v1, interval<float> v2 ) {
    return __ri_ceq( v1, v2 );
}
inline interval_bool cne( interval<float> v1, interval<float> v2 ) {
    return __ri_cne( v1, v2 );
}
inline interval_bool clt( interval<float> v1, interval<float> v2 ) {
    return __ri_clt( v1, v2 );
}
inline interval_bool cle( interval<float> v1, interval<float> v2 ) {
    return __ri_cle( v1, v2 );
}
inline interval_bool cgt( interval<float> v1, interval<float> v2 ) {
    return __ri_cgt( v1, v2 );
}
inline interval_bool cge( interval<float> v1, interval<float> v2 ) {
    return __ri_cge( v1, v2 );
}

/* Possibly Relations: equal, not equal, less than, less than or equal,
greater than, greater than or equal */

inline interval_bool peq( interval<float> v1, interval<float> v2 ) {
    return __ri_peq( v1, v2 );
}
inline interval_bool pne( interval<float> v1, interval<float> v2 ) {
    return __ri_pne( v1, v2 );
}
inline interval_bool plt( interval<float> v1, interval<float> v2 ) {
    return __ri_plt( v1, v2 );
}
inline interval_bool ple( interval<float> v1, interval<float> v2 ) {
    return __ri_ple( v1, v2 );
}
inline interval_bool pgt( interval<float> v1, interval<float> v2 ) {
    return __ri_pgt( v1, v2 );
}
inline interval_bool pge( interval<float> v1, interval<float> v2 ) {
    return __ri_pge( v1, v2 );
}

/* Arithmetic Functions: absolute value, min, max, ceil, floor, fmod */

inline interval<float> fabs( interval<float> v ) {
    return __ri_fabs( v );
}
inline interval<float> minimum( interval<float> v1, interval<float> v2 ) {
    return __ri_min( v1, v2 );
}
inline interval<float> maximum( interval<float> v1, interval<float> v2 ) {
    return __ri_max( v1, v2 );
}
inline double ceil( interval<float> v ) {
    return ::ceil( v.sup() );
}
inline double floor( interval<float> v ) {
    return ::floor( v.inf() );
}
inline interval<float> fmod( interval<float> v1, interval<float> v2 ) {
    return __ri_fmod( v1, v2 );
}

/* Power Functions: square, square root, cube root, natural power,
natural log, log base 10, integral power, floating power */

inline interval<float> sqr( interval<float> v ) {
    return __ri_sqr( v );
}
inline interval<float> sqrt( interval<float> v ) {
    return __ri_sqrt( v );
}
inline interval<float> cbrt( interval<float> v ) {
    return __ri_cbrt( v );
}
inline interval<float> exp( interval<float> v ) {
    return __ri_exp( v );
}
inline interval<float> log( interval<float> v ) {
    return __ri_log( v );
}
inline interval<float> log10( interval<float> v ) {
    return __ri_log10( v );
}
inline interval<float> pow( interval<float> v1, int v2 ) {
    return __ri_pow_i( v1, v2 );
}
inline interval<float> pow( interval<float> v1, interval<float> v2 ) {
    return __ri_pow( v1, v2 );
}

/* Trigonometric Functions */

inline interval<float> sin( interval<float> v ) {
    return __ri_sin( v );
}
inline interval<float> sinh( interval<float> v ) {
    return __ri_sinh( v );
}
inline interval<float> asin( interval<float> v ) {
    return __ri_asin( v );
}
inline interval<float> cos( interval<float> v ) {
    return __ri_cos( v );
}
inline interval<float> cosh( interval<float> v ) {
    return __ri_cosh( v );
}
inline interval<float> acos( interval<float> v ) {
    return __ri_acos( v );
}
inline interval<float> tan( interval<float> v ) {
    return __ri_tan( v );
}
inline interval<float> tanh( interval<float> v ) {
    return __ri_tanh( v );
}
inline interval<float> atan( interval<float> v ) {
    return __ri_atan( v );
}
inline interval<float> atan2( interval<float> v1, interval<float> v2 ) {
    return __ri_atan2( v1, v2 );
}

/* Input/Output Functions */

ostream& operator << ( ostream&, interval<float> );
void single_number_output( interval<float> x, ostream& out = cout );
istream& operator >> ( istream&, interval<float>& );

/*****************************************************************************/

template<>
struct interval<double> : __d_interval
{
    /* Constructors */

    interval() { }

    interval( __d_interval v ) : __d_interval( v ) { }

    explicit interval( const interval<float>& v ) {
        __di_interval_ri_( (__d_interval*)this, (__r_interval*)&v );
    }
    //       interval( const interval<double>& ) is default
    explicit interval( const interval<long double>& v ) {
        __di_interval_qi_( (__d_interval*)this, (__q_interval*)&v );
    }

    explicit interval( int v ) {
        __di_interval_i_( (__d_interval*)this, v );
    }
    explicit interval( long long v ) {
        __di_interval_xl_( (__d_interval*)this, v );
    }
    explicit interval( float v ) {
        __di_interval_r_( (__d_interval*)this, v );
    }
    explicit interval( double v ) {
        __di_interval_d_( (__d_interval*)this, v );
    }
    explicit interval( long double v ) {
        __di_interval_q_( (__d_interval*)this, v );
    }

    interval( int v1, int v2 ) {
        __di_interval_i_i_( (__d_interval*)this, v1, v2 );
    }
    interval( long long v1, long long v2 ) {
        __di_interval_xl_xl_( (__d_interval*)this, v1, v2 );
    }
    interval( float v1, float v2 ) {
        __di_interval_r_r_( (__d_interval*)this, v1, v2 );
    }
    interval( double v1, double v2 ) {
        __di_interval_d_d_( (__d_interval*)this, v1, v2 );
    }
    interval( long double v1, long double v2 ) {
        __di_interval_q_q_( (__d_interval*)this, v1, v2 );
    }

    explicit interval( const char* );

    interval<double>& operator += ( interval<double> v) {
        *this = __di_add( *this, v );
        return *this;
    }
    interval<double>& operator -= ( interval<double> v ) {
        *this = __di_sub( *this, v );
        return *this;
    }
    interval<double>& operator *= ( interval<double> v ) {
        *this = __di_mul( *this, v );
        return *this;
    }
    interval<double>& operator /= ( interval<double> v ) {
        *this = __di_div( *this, v );
        return *this;
    }

    /* Extraction Functions: infimim, supremum */

    double inf() const { return l; }
    double sup() const { return u; }
};

/* Extraction Functions: infimim, supremum, midpoint, width, magnitude,
mignitude */

inline double inf( interval<double> v ) { return v.inf(); }
inline double sup( interval<double> v ) { return v.sup(); }
inline double mid( interval<double> v ) { return __di_mid( v ); }
inline double wid( interval<double> v ) { return __di_wid( v ); }
inline double mag( interval<double> v ) { return __di_mag( v ); }
inline double mig( interval<double> v ) { return __di_mig( v ); }

/* Arithmetic Operations: addition, subtraction, multiplication,
division, negation */

inline interval<double>
operator + ( interval<double> v1, interval<double> v2 ) {
    return __di_add( v1, v2 );
}
inline interval<double>
operator - ( interval<double> v1, interval<double> v2 ) {
    return __di_sub( v1, v2 );
}
inline interval<double>
operator * ( interval<double> v1, interval<double> v2 ) {
    return __di_mul( v1, v2 );
}
inline interval<double>
operator / ( interval<double> v1, interval<double> v2 ) {
    return __di_div( v1, v2 );
}
inline interval<double>
operator - ( interval<double> v ) {
    return __di_neg( v );
}

/* Intersection Division */

inline interval<double> 
divix( interval<double> v1, interval<double> v2, interval<double> v3 ) {
    return __di_divx( v1, v2, v3 );
}

/* Hull Functions: regular hull, intersection */

inline interval<double>
interval_hull( interval<double> v1, interval<double> v2 ) {
    return __di_ih( v1, v2 );
}
inline interval<double> intersect( interval<double> v1, interval<double> v2 ) {
    return __di_ix( v1, v2 );
}

/* Query Functions: is empty, ndigits */

inline interval_bool is_empty( interval<double> v ) {
    return __di_isempty( v );
}
inline int ndigits( interval<double> v ) {
    return __di_ndigits( v );
}

/* Membership Functions */

inline interval_bool in( int c, interval<double> v ) {
    return __i_in_di( c, v );
}
inline interval_bool in( long long c, interval<double> v ) {
    return __xl_in_di( c, v );
}
inline interval_bool in( float c, interval<double> v ) {
    return __r_in_di( c, v );
}
inline interval_bool in( double c, interval<double> v ) {
    return __d_in_di( c, v );
}
inline interval_bool in( long double c, interval<double> v ) {
    return __q_in_di( c, v );
}

/* Comparison Functions: disjoint, in interior, subset, superset,
proper subset, proper superset */

inline interval_bool disjoint( interval<double> v1, interval<double> v2 ) {
    return __di_dj( v1, v2 );
}
inline interval_bool in_interior( interval<double> v1, interval<double> v2 ) {
    return __di_int( v1, v2 );
}
inline interval_bool subset( interval<double> v1, interval<double> v2 ) {
    return __di_sb( v1, v2 );
}
inline interval_bool superset( interval<double> v1, interval<double> v2 ) {
    return __di_sp( v1, v2 );
}
inline interval_bool proper_subset( interval<double> v1, interval<double> v2 ) {
    return __di_psb( v1, v2 );
}
inline interval_bool
proper_superset( interval<double> v1, interval<double> v2 ) {
    return __di_psp( v1, v2 );
}

/* Set Relations: equal, not equal, less than, less than or equal,
greater than, greater than or equal */

inline interval_bool seq( interval<double> v1, interval<double> v2 ) {
    return __di_seq( v1, v2 );
}
inline interval_bool operator ==( interval<double> v1, interval<double> v2 ) {
    return __di_seq( v1, v2 );
}
inline interval_bool sne( interval<double> v1, interval<double> v2 ) {
    return __di_sne( v1, v2 );
}
inline interval_bool operator !=( interval<double> v1, interval<double> v2 ) {
    return __di_sne( v1, v2 );
}
inline interval_bool slt( interval<double> v1, interval<double> v2 ) {
    return __di_slt( v1, v2 );
}
inline interval_bool sle( interval<double> v1, interval<double> v2 ) {
    return __di_sle( v1, v2 );
}
inline interval_bool sgt( interval<double> v1, interval<double> v2 ) {
    return __di_sgt( v1, v2 );
}
inline interval_bool sge( interval<double> v1, interval<double> v2 ) {
    return __di_sge( v1, v2 );
}

/* Certainly Relations: equal, not equal, less than, less than or
equal, greater than, greater than or equal */

inline interval_bool ceq( interval<double> v1, interval<double> v2 ) {
    return __di_ceq( v1, v2 );
}
inline interval_bool cne( interval<double> v1, interval<double> v2 ) {
    return __di_cne( v1, v2 );
}
inline interval_bool clt( interval<double> v1, interval<double> v2 ) {
    return __di_clt( v1, v2 );
}
inline interval_bool cle( interval<double> v1, interval<double> v2 ) {
    return __di_cle( v1, v2 );
}
inline interval_bool cgt( interval<double> v1, interval<double> v2 ) {
    return __di_cgt( v1, v2 );
}
inline interval_bool cge( interval<double> v1, interval<double> v2 ) {
    return __di_cge( v1, v2 );
}

/* Possibly Relations: equal, not equal, less than, less than or equal,
greater than, greater than or equal */

inline interval_bool peq( interval<double> v1, interval<double> v2 ) {
    return __di_peq( v1, v2 );
}
inline interval_bool pne( interval<double> v1, interval<double> v2 ) {
    return __di_pne( v1, v2 );
}
inline interval_bool plt( interval<double> v1, interval<double> v2 ) {
    return __di_plt( v1, v2 );
}
inline interval_bool ple( interval<double> v1, interval<double> v2 ) {
    return __di_ple( v1, v2 );
}
inline interval_bool pgt( interval<double> v1, interval<double> v2 ) {
    return __di_pgt( v1, v2 );
}
inline interval_bool pge( interval<double> v1, interval<double> v2 ) {
    return __di_pge( v1, v2 );
}

/* Arithmetic Functions: absolute value, min, max, ceil, floor, fmod */

inline interval<double> fabs( interval<double> v ) {
    return __di_fabs( v );
}
inline interval<double> minimum( interval<double> v1, interval<double> v2 ) {
    return __di_min( v1, v2 );
}
inline interval<double> maximum( interval<double> v1, interval<double> v2 ) {
    return __di_max( v1, v2 );
}
inline double ceil( interval<double> v ) {
    return ::ceil( v.sup() );
}
inline double floor( interval<double> v ) {
    return ::floor( v.inf() );
}
inline interval<double> fmod( interval<double> v1, interval<double> v2 ) {
    return __di_fmod( v1, v2 );
}

/* Power Functions: square, square root, cube root, natural power,
natural log, log base 10, integral power, floating power */

inline interval<double> sqr( interval<double> v ) {
    return __di_sqr( v );
}
inline interval<double> sqrt( interval<double> v ) {
    return __di_sqrt( v );
}
inline interval<double> cbrt( interval<double> v ) {
    return __di_cbrt( v );
}
inline interval<double> exp( interval<double> v ) {
    return __di_exp( v );
}
inline interval<double> log( interval<double> v ) {
    return __di_log( v );
}
inline interval<double> log10( interval<double> v ) {
    return __di_log10( v );
}
inline interval<double> pow( interval<double> v1, int v2 ) {
    return __di_pow_i( v1, v2 );
}
inline interval<double> pow( interval<double> v1, interval<double> v2 ) {
    return __di_pow( v1, v2 );
}

/* Trigonometric Functions */

inline interval<double> sin( interval<double> v ) {
    return __di_sin( v );
}
inline interval<double>  sinh( interval<double> v ) {
    return __di_sinh( v );
}
inline interval<double> asin( interval<double> v ) {
    return __di_asin( v );
}
inline interval<double> cos( interval<double> v ) {
    return __di_cos( v );
}
inline interval<double> cosh( interval<double> v ) {
    return __di_cosh( v );
}
inline interval<double> acos( interval<double> v ) {
    return __di_acos( v );
}
inline interval<double> tan( interval<double> v ) {
    return __di_tan( v );
}
inline interval<double> tanh( interval<double> v ) {
    return __di_tanh( v );
}
inline interval<double> atan( interval<double> v ) {
    return __di_atan( v );
}
inline interval<double> atan2( interval<double> v1, interval<double> v2 ) {
    return __di_atan2( v1, v2 );
}

/* Input/Output Functions */

ostream& operator << ( ostream&, interval<double> );
void single_number_output( interval<double> x, ostream& out = cout );
istream& operator >> ( istream&, interval<double>& );

/*****************************************************************************/

template<>
struct interval<long double> : __q_interval
{
    /* Constructors */

    interval() { }

    interval( __q_interval v ) : __q_interval( v ) { }

    explicit interval( const interval<float>& v ) {
        __qi_interval_ri_( (__q_interval*)this, (__r_interval*)&v );
    }
    explicit interval( const interval<double>& v ) {
        __qi_interval_di_( (__q_interval*)this, (__d_interval*)&v );
    }
    //       interval( const interval<long double>& ) is default

    explicit interval( int v ) {
        __qi_interval_i_( (__q_interval*)this, v );
    }
    explicit interval( long long v ) {
        __qi_interval_xl_( (__q_interval*)this, v );
    }
    explicit interval( float v ) {
        __qi_interval_r_( (__q_interval*)this, v );
    }
    explicit interval( double v ) {
        __qi_interval_d_( (__q_interval*)this, v );
    }
    explicit interval( long double v ) {
        __qi_interval_q_( (__q_interval*)this, v );
    }

    interval( int v1, int v2 ) {
        __qi_interval_i_i_( (__q_interval*)this, v1, v2 );
    }
    interval( long long v1, long long v2 ) {
        __qi_interval_xl_xl_( (__q_interval*)this, v1, v2 );
    }
    interval( float v1, float v2 ) {
        __qi_interval_r_r_( (__q_interval*)this, v1, v2 );
    }
    interval( double v1, double v2 ) {
        __qi_interval_d_d_( (__q_interval*)this, v1, v2 );
    }
    interval( long double v1, long double v2 ) {
        __qi_interval_q_q_( (__q_interval*)this, v1, v2 );
    }

    explicit interval( const char* );

    interval<long double>& operator += ( interval<long double> v) {
        *this = __qi_add( *this, v );
        return *this;
    }
    interval<long double>& operator -= ( interval<long double> v ) {
        *this = __qi_sub( *this, v );
        return *this;
    }
    interval<long double>& operator *= ( interval<long double> v ) {
        *this = __qi_mul( *this, v );
        return *this;
    }
    interval<long double>& operator /= ( interval<long double> v ) {
        *this = __qi_div( *this, v );
        return *this;
    }

    /* Extraction Functions: infimim, supremum */

    long double inf() const { return l; }
    long double sup() const { return u; }
};

/* Extraction Functions: infimim, supremum, midpoint, width, magnitude,
mignitude */

inline long double inf( interval<long double> v ) { return v.inf(); }
inline long double sup( interval<long double> v ) { return v.sup(); }
inline long double mid( interval<long double> v ) { return __qi_mid( v ); }
inline long double wid( interval<long double> v ) { return __qi_wid( v ); }
inline long double mag( interval<long double> v ) { return __qi_mag( v ); }
inline long double mig( interval<long double> v ) { return __qi_mig( v ); }

/* Arithmetic Operations: addition, subtraction, multiplication,
division, negation */

inline interval<long double>
operator + ( interval<long double> v1, interval<long double> v2 ) {
    return __qi_add( v1, v2 );
}
inline interval<long double>
operator - ( interval<long double> v1, interval<long double> v2 ) {
    return __qi_sub( v1, v2 );
}
inline interval<long double>
operator * ( interval<long double> v1, interval<long double> v2 ) {
    return __qi_mul( v1, v2 );
}
inline interval<long double>
operator / ( interval<long double> v1, interval<long double> v2 ) {
    return __qi_div( v1, v2 );
}
inline interval<long double> operator - ( interval<long double> v ) {
    return __qi_neg( v );
}

/* Intersection Division */

inline interval<long double> 
divix( interval<long double> v1, interval<long double> v2,
       interval<long double> v3) {
    return __qi_divx( v1, v2, v3 );
}

/* Hull Functions: interval hull, intersection */

inline interval<long double>
interval_hull( interval<long double> v1, interval<long double> v2 ) {
    return __qi_ih( v1, v2 );
}
inline interval<long double>
intersect( interval<long double> v1, interval<long double> v2 ) {
    return __qi_ix( v1, v2 );
}

/* Query Functions: is empty, ndigits */

inline interval_bool is_empty( interval<long double> v ) {
    return __qi_isempty( v );
}
inline int ndigits( interval<long double> v ) {
    return __qi_ndigits( v );
}

/* Membership Functions */

inline interval_bool in( int c, interval<long double> v ) {
    return __i_in_qi( c, v );
}
inline interval_bool in( long long c, interval<long double> v ) {
    return __xl_in_qi( c, v );
}
inline interval_bool in( float c, interval<long double> v ) {
    return __r_in_qi( c, v );
}
inline interval_bool in( double c, interval<long double> v ) {
    return __d_in_qi( c, v );
}
inline interval_bool in( long double c, interval<long double> v ) {
    return __q_in_qi( c, v );
}

/* Comparison Functions: disjoint, in interior, subset, superset,
proper subset, proper superset */

inline interval_bool
disjoint( interval<long double> v1, interval<long double> v2 ) {
    return __qi_dj( v1, v2 );
}
inline interval_bool
in_interior( interval<long double> v1, interval<long double> v2 ) {
    return __qi_int( v1, v2 );
}
inline interval_bool
subset( interval<long double> v1, interval<long double> v2 ) {
    return __qi_sb( v1, v2 );
}
inline interval_bool
superset( interval<long double> v1, interval<long double> v2 ) {
    return __qi_sp( v1, v2 );
}
inline interval_bool
proper_subset( interval<long double> v1, interval<long double> v2 ) {
    return __qi_psb( v1, v2 );
}
inline interval_bool
proper_superset( interval<long double> v1, interval<long double> v2 ) {
    return __qi_psp( v1, v2 );
}

/* Set Relations: equal, not equal, less than, less than or equal,
greater than, greater than or equal */

inline interval_bool seq( interval<long double> v1, interval<long double> v2 ) {
    return __qi_seq( v1, v2 );
}
inline interval_bool
operator ==( interval<long double> v1, interval<long double> v2 ) {
    return __qi_seq( v1, v2 );
}
inline interval_bool sne( interval<long double> v1, interval<long double> v2 ) {
    return __qi_sne( v1, v2 );
}
inline interval_bool
operator !=( interval<long double> v1, interval<long double> v2 ) {
    return __qi_sne( v1, v2 );
}
inline interval_bool slt( interval<long double> v1, interval<long double> v2 ) {
    return __qi_slt( v1, v2 );
}
inline interval_bool sle( interval<long double> v1, interval<long double> v2 ) {
    return __qi_sle( v1, v2 );
}
inline interval_bool sgt( interval<long double> v1, interval<long double> v2 ) {
    return __qi_sgt( v1, v2 );
}
inline interval_bool sge( interval<long double> v1, interval<long double> v2 ) {
    return __qi_sge( v1, v2 );
}

/* Certainly Relations: equal, not equal, less than, less than or
equal, greater than, greater than or equal */

inline interval_bool ceq( interval<long double> v1, interval<long double> v2 ) {
    return __qi_ceq( v1, v2 );
}
inline interval_bool cne( interval<long double> v1, interval<long double> v2 ) {
    return __qi_cne( v1, v2 );
}
inline interval_bool clt( interval<long double> v1, interval<long double> v2 ) {
    return __qi_clt( v1, v2 );
}
inline interval_bool cle( interval<long double> v1, interval<long double> v2 ) {
    return __qi_cle( v1, v2 );
}
inline interval_bool cgt( interval<long double> v1, interval<long double> v2 ) {
    return __qi_cgt( v1, v2 );
}
inline interval_bool cge( interval<long double> v1, interval<long double> v2 ) {
    return __qi_cge( v1, v2 );
}

/* Possibly Relations: equal, not equal, less than, less than or equal,
greater than, greater than or equal */

inline interval_bool peq( interval<long double> v1, interval<long double> v2 ) {
    return __qi_peq( v1, v2 );
}
inline interval_bool pne( interval<long double> v1, interval<long double> v2 ) {
    return __qi_pne( v1, v2 );
}
inline interval_bool plt( interval<long double> v1, interval<long double> v2 ) {
    return __qi_plt( v1, v2 );
}
inline interval_bool ple( interval<long double> v1, interval<long double> v2 ) {
    return __qi_ple( v1, v2 );
}
inline interval_bool pgt( interval<long double> v1, interval<long double> v2 ) {
    return __qi_pgt( v1, v2 );
}
inline interval_bool pge( interval<long double> v1, interval<long double> v2 ) {
    return __qi_pge( v1, v2 );
}

/* Arithmetic Functions: absolute value, min, max, ceil, floor, fmod */

inline interval<long double> 
fabs( interval<long double> v ) {
    return __qi_fabs( v );
}
inline interval<long double>
minimum( interval<long double> v1, interval<long double> v2 ) {
    return __qi_min( v1, v2 );
}
inline interval<long double>
maximum( interval<long double> v1, interval<long double> v2 ) {
    return __qi_max( v1, v2 );
}
inline double ceil( interval<long double> v ) {
    return ::ceil( v.sup() );
}
inline double floor( interval<long double> v ) {
    return ::floor( v.inf() );
}
inline interval<long double>
fmod( interval<long double> v1, interval<long double> v2 ) {
    return __qi_fmod( v1, v2 );
}

/* Power Functions: integral power, floating power */

inline interval<long double>
pow( interval<long double> v1, long long v2 ) {
    return __qi_pow_xl( v1, v2 );
}
inline interval<long double>
pow( interval<long double> v1, interval<long double> v2 ) {
    return __qi_pow( v1, v2 );
}
inline interval<long double> sqrt( interval<long double> v ) {
    return __qi_sqrt( v );
}
inline interval<long double> exp( interval<long double> v ) {
    return __qi_exp( v );
}
inline interval<long double> log( interval<long double> v ) {
    return __qi_log( v );
}
inline interval<long double> sin( interval<long double> v ) {
    return __qi_sin( v );
}
inline interval<long double> cos( interval<long double> v ) {
    return __qi_cos( v );
}
inline interval<long double> asin( interval<long double> v ) {
    return __qi_asin( v );
}
inline interval<long double> acos( interval<long double> v ) {
    return __qi_acos( v );
}

/* Input/Output Functions */

ostream& operator << ( ostream&, interval<long double> );
void single_number_output( interval<long double> x, ostream& out = cout );
istream& operator >> ( istream&, interval<long double>& );

/*****************************************************************************/

#if __cplusplus >= 199711
} // namespace SUNW_interval
#endif

#endif	/* !defined(__SUNINTERVAL_H) */
