#ifndef __SUNINTERVAL_VALARRAY_H
#define __SUNINTERVAL_VALARRAY_H

#include <valarray>
#include <suninterval.h>

namespace SUNW_interval {

template< typename T >
T sum( const std::valarray<T>& a );

template< typename T >
T product( const std::valarray<T>& a );

template< typename T >
std::valarray<T> inf( const std::valarray< interval<T> >& a );

template< typename T >
std::valarray<T> sup( const std::valarray< interval<T> >& a );

template< typename T >
std::valarray<T> wid( const std::valarray< interval<T> >& a );

template< typename T >
std::valarray<T> mid( const std::valarray< interval<T> >& a );

template< typename T >
std::valarray<T> mag( const std::valarray< interval<T> >& a );

template< typename T >
std::valarray<T> mig( const std::valarray< interval<T> >& a );

template< typename T >
std::valarray<T> ceil( const std::valarray< interval<T> >& a );

template< typename T >
std::valarray<T> floor( const std::valarray< interval<T> >& a );

template< typename T >
std::valarray<T> 
divix( const std::valarray<T>& a, const std::valarray<T>& b, 
       const std::valarray<T>& c );

template< typename T >
std::valarray<T> 
interval_hull( const std::valarray<T>& a, const std::valarray<T>& b );

template< typename T >
std::valarray<T> 
intersect( const std::valarray<T>& a, const std::valarray<T>& b );

template< typename T >
std::valarray<T> 
interval_hull( const std::valarray<T>& a, const T& b );

template< typename T >
std::valarray<T> 
intersect( const std::valarray<T>& a, const T& b );

template< typename T >
std::valarray<T> 
interval_hull( const T& a, const std::valarray<T>& b );

template< typename T >
std::valarray<T> 
intersect( const T& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
is_empty( const std::valarray<T>& a );

template< typename T >
std::valarray<int> 
ndigits( const std::valarray<T>& a );

template< typename T1, typename T2 >
std::valarray<interval_bool> 
in( const std::valarray<T1>& a, const std::valarray<T2>& b );

template< typename T >
std::valarray<interval_bool> 
disjoint( const std::valarray<T>& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
in_interior( const std::valarray<T>& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
subset( const std::valarray<T>& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
superset( const std::valarray<T>& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
proper_subset( const std::valarray<T>& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
proper_superset( const std::valarray<T>& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
seq( const std::valarray<T>& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
sne( const std::valarray<T>& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
slt( const std::valarray<T>& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
sle( const std::valarray<T>& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
sgt( const std::valarray<T>& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
sge( const std::valarray<T>& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
ceq( const std::valarray<T>& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
cne( const std::valarray<T>& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
clt( const std::valarray<T>& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
cle( const std::valarray<T>& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
cgt( const std::valarray<T>& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
cge( const std::valarray<T>& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
peq( const std::valarray<T>& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
pne( const std::valarray<T>& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
plt( const std::valarray<T>& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
ple( const std::valarray<T>& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
pgt( const std::valarray<T>& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
pge( const std::valarray<T>& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
disjoint( const std::valarray<T>& a, const T& b );

template< typename T >
std::valarray<interval_bool> 
in_interior( const std::valarray<T>& a, const T& b );

template< typename T >
std::valarray<interval_bool> 
subset( const std::valarray<T>& a, const T& b );

template< typename T >
std::valarray<interval_bool> 
superset( const std::valarray<T>& a, const T& b );

template< typename T >
std::valarray<interval_bool> 
proper_subset( const std::valarray<T>& a, const T& b );

template< typename T >
std::valarray<interval_bool> 
proper_superset( const std::valarray<T>& a, const T& b );

template< typename T >
std::valarray<interval_bool> 
seq( const std::valarray<T>& a, const T& b );

template< typename T >
std::valarray<interval_bool> 
sne( const std::valarray<T>& a, const T& b );

template< typename T >
std::valarray<interval_bool> 
slt( const std::valarray<T>& a, const T& b );

template< typename T >
std::valarray<interval_bool> 
sle( const std::valarray<T>& a, const T& b );

template< typename T >
std::valarray<interval_bool> 
sgt( const std::valarray<T>& a, const T& b );

template< typename T >
std::valarray<interval_bool> 
sge( const std::valarray<T>& a, const T& b );

template< typename T >
std::valarray<interval_bool> 
ceq( const std::valarray<T>& a, const T& b );

template< typename T >
std::valarray<interval_bool> 
cne( const std::valarray<T>& a, const T& b );

template< typename T >
std::valarray<interval_bool> 
clt( const std::valarray<T>& a, const T& b );

template< typename T >
std::valarray<interval_bool> 
cle( const std::valarray<T>& a, const T& b );

template< typename T >
std::valarray<interval_bool> 
cgt( const std::valarray<T>& a, const T& b );

template< typename T >
std::valarray<interval_bool> 
cge( const std::valarray<T>& a, const T& b );

template< typename T >
std::valarray<interval_bool> 
peq( const std::valarray<T>& a, const T& b );

template< typename T >
std::valarray<interval_bool> 
pne( const std::valarray<T>& a, const T& b );

template< typename T >
std::valarray<interval_bool> 
plt( const std::valarray<T>& a, const T& b );

template< typename T >
std::valarray<interval_bool> 
ple( const std::valarray<T>& a, const T& b );

template< typename T >
std::valarray<interval_bool> 
pgt( const std::valarray<T>& a, const T& b );

template< typename T >
std::valarray<interval_bool> 
pge( const std::valarray<T>& a, const T& b );

template< typename T >
std::valarray<interval_bool> 
disjoint( const T& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
in_interior( const T& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
subset( const T& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
superset( const T& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
proper_subset( const T& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
proper_superset( const T& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
seq( const T& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
sne( const T& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
slt( const T& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
sle( const T& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
sgt( const T& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
sge( const T& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
ceq( const T& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
cne( const T& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
clt( const T& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
cle( const T& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
cgt( const T& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
cge( const T& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
peq( const T& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
pne( const T& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
plt( const T& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
ple( const T& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
pgt( const T& a, const std::valarray<T>& b );

template< typename T >
std::valarray<interval_bool> 
pge( const T& a, const std::valarray<T>& b );

template< typename T >
std::valarray<T> 
fabs( const std::valarray<T>& a );

template< typename T >
std::valarray<T> 
minimum( const std::valarray<T>& a, const std::valarray<T>& b );

template< typename T >
std::valarray<T> 
maximum( const std::valarray<T>& a, const std::valarray<T>& b );

template< typename T >
std::valarray<T> 
fmod( const std::valarray<T>& a, const std::valarray<T>& b );

template< typename T >
std::valarray<T> 
minimum( const std::valarray<T>& a, const T& b );

template< typename T >
std::valarray<T> 
maximum( const std::valarray<T>& a, const T& b );

template< typename T >
std::valarray<T> 
fmod( const std::valarray<T>& a, const T& b );

template< typename T >
std::valarray<T> 
minimum( const T& a, const std::valarray<T>& b );

template< typename T >
std::valarray<T> 
maximum( const T& a, const std::valarray<T>& b );

template< typename T >
std::valarray<T> 
fmod( const T& a, const std::valarray<T>& b );

template< typename T >
std::valarray<T> 
sqr( const std::valarray<T>& a );

template< typename T >
std::valarray<T> 
cbrt( const std::valarray<T>& a );

template< typename T >
std::valarray<T> 
pow( const std::valarray<T>& a, const std::valarray<int>& b );

template< typename T >
std::valarray<T> 
pow( const std::valarray<T>& a, const int& b );

template< typename T >
std::valarray<T> 
pow( const T& a, const std::valarray<int>& b );

template< typename T >
std::valarray< interval<T> > 
operator+ ( const std::valarray< interval<T> >& a,
            const std::valarray< interval<T> >& b );

template< typename T >
std::valarray< interval<T> > 
operator- ( const std::valarray< interval<T> >& a,
            const std::valarray< interval<T> >& b );

template< typename T >
std::valarray< interval<T> >
operator* ( const std::valarray< interval<T> >& a,
            const std::valarray< interval<T> >& b );

template< typename T >
std::valarray< interval<T> >
operator/ ( const std::valarray< interval<T> >& a,
            const std::valarray< interval<T> >& b );

template< typename T >
std::valarray< interval<T> > 
operator+ ( const std::valarray< interval<T> >& a,
            const interval<T>& b );

template< typename T >
std::valarray< interval<T> > 
operator- ( const std::valarray< interval<T> >& a,
            const interval<T>& b );

template< typename T >
std::valarray< interval<T> >
operator* ( const std::valarray< interval<T> >& a,
            const interval<T>& b );

template< typename T >
std::valarray< interval<T> >
operator/ ( const std::valarray< interval<T> >& a,
            const interval<T>& b );

template< typename T >
std::valarray< interval<T> > 
operator+ ( const interval<T>& a,
            const std::valarray< interval<T> >& b );

template< typename T >
std::valarray< interval<T> > 
operator- ( const interval<T>& a,
            const std::valarray< interval<T> >& b );

template< typename T >
std::valarray< interval<T> >
operator* ( const interval<T>& a,
            const std::valarray< interval<T> >& b );

template< typename T >
std::valarray< interval<T> >
operator/ ( const interval<T>& a,
            const std::valarray< interval<T> >& b );

} // namespace SUNW_interval
    
namespace std {

template<>
valarray< SUNW_interval::interval<float> >&
valarray< SUNW_interval::interval<float> >::operator+= 
        ( const valarray< SUNW_interval::interval<float> >& a );

template<>
valarray< SUNW_interval::interval<double> >&
valarray< SUNW_interval::interval<double> >::operator+= 
        ( const valarray< SUNW_interval::interval<double> >& a );

template<>
valarray< SUNW_interval::interval<long double> >&
valarray< SUNW_interval::interval<long double> >::operator+= 
        ( const valarray< SUNW_interval::interval<long double> >& a );

template<>
valarray< SUNW_interval::interval<float> >&
valarray< SUNW_interval::interval<float> >::operator-= 
        ( const valarray< SUNW_interval::interval<float> >& a );
        
template<>
valarray< SUNW_interval::interval<double> >&
valarray< SUNW_interval::interval<double> >::operator-= 
        ( const valarray< SUNW_interval::interval<double> >& a );

template<>
valarray< SUNW_interval::interval<long double> >&
valarray< SUNW_interval::interval<long double> >::operator-= 
        ( const valarray< SUNW_interval::interval<long double> >& a );
    
template<>
valarray< SUNW_interval::interval<float> >&
valarray< SUNW_interval::interval<float> >::operator*= 
        ( const valarray< SUNW_interval::interval<float> >& a );

template<>
valarray< SUNW_interval::interval<double> >&
valarray< SUNW_interval::interval<double> >::operator*= 
        ( const valarray< SUNW_interval::interval<double> >& a );

template<>
valarray< SUNW_interval::interval<long double> >&
valarray< SUNW_interval::interval<long double> >::operator*= 
        ( const valarray< SUNW_interval::interval<long double> >& a );
    
template<>
valarray< SUNW_interval::interval<float> >&
valarray< SUNW_interval::interval<float> >::operator/= 
        ( const valarray< SUNW_interval::interval<float> >& a );

template<>
valarray< SUNW_interval::interval<double> >&
valarray< SUNW_interval::interval<double> >::operator/= 
        ( const valarray< SUNW_interval::interval<double> >& a );

template<>
valarray< SUNW_interval::interval<long double> >&
valarray< SUNW_interval::interval<long double> >::operator/= 
        ( const valarray< SUNW_interval::interval<long double> >& a );
    
template<>
valarray< SUNW_interval::interval<float> >& 
valarray< SUNW_interval::interval<float> >::operator+= 
        ( const SUNW_interval::interval<float>& a );

template<>
valarray< SUNW_interval::interval<double> >& 
valarray< SUNW_interval::interval<double> >::operator+= 
        ( const SUNW_interval::interval<double>& a );
        
template<>
valarray< SUNW_interval::interval<long double> >& 
valarray< SUNW_interval::interval<long double> >::operator+= 
        ( const SUNW_interval::interval<long double>& a );

template<>
valarray< SUNW_interval::interval<float> >& 
valarray< SUNW_interval::interval<float> >::operator-= 
        ( const SUNW_interval::interval<float>& a );
        
template<>
valarray< SUNW_interval::interval<double> >& 
valarray< SUNW_interval::interval<double> >::operator-= 
        ( const SUNW_interval::interval<double>& a );
        
template<>
valarray< SUNW_interval::interval<long double> >& 
valarray< SUNW_interval::interval<long double> >::operator-= 
        ( const SUNW_interval::interval<long double>& a );
        
template<>
valarray< SUNW_interval::interval<float> >& 
valarray< SUNW_interval::interval<float> >::operator*= 
        ( const SUNW_interval::interval<float>& a );
        
template<>
valarray< SUNW_interval::interval<double> >& 
valarray< SUNW_interval::interval<double> >::operator*= 
        ( const SUNW_interval::interval<double>& a );
        
template<>
valarray< SUNW_interval::interval<long double> >& 
valarray< SUNW_interval::interval<long double> >::operator*= 
        ( const SUNW_interval::interval<long double>& a );

template<>
valarray< SUNW_interval::interval<float> >& 
valarray< SUNW_interval::interval<float> >::operator/= 
        ( const SUNW_interval::interval<float>& a );
        
template<>
valarray< SUNW_interval::interval<double> >& 
valarray< SUNW_interval::interval<double> >::operator/= 
        ( const SUNW_interval::interval<double>& a );
        
template<>
valarray< SUNW_interval::interval<long double> >& 
valarray< SUNW_interval::interval<long double> >::operator/= 
        ( const SUNW_interval::interval<long double>& a );

} // namespace std

#endif /* !defined(__SUNINTERVAL_VALARRAY_H) */
